/***************************************************************************
                          canvasshapes.h  -  description
                             -------------------
    begin                : Tue Jul 30 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef ROUNDEDRECTANGLE_H
#define ROUNDEDRECTANGLE_H

/** \class CanvasRoundedRectangle
  * \brief A descendent of QCanvasPolygon for rectangles with rounded corners.
  * @author Ryan Rusaw
  */
class CanvasRoundedRectangle : public QCanvasPolygon
{
public:
    /**
      * \brief Constructor.
      * @param x x value of top left corner of bounding rectangle on canvas.
      * @param y y value of top left corner of bounding rectangle on canvas.
      * @param w width of bounding rectangle
      * @param h height of bounding rectangle
      * @param rad radius of rounded corners
      * @param canvas canvas containing for item
      */
     CanvasRoundedRectangle(int x,int y,int w,int h,int rad, QCanvas *canvas);
     /**
       * \brief Deconstructor
       */
     ~CanvasRoundedRectangle() { this->hide(); };
     /**
       * \brief Runtime typing value
       * @return Runtime typing value
       */
     int rtti() const;
     /**
       * \brief Gets internal outline points array
       * @return outlinepoints
       */
     QPointArray outlinePoints() { return outlinepoints; };
private:
     /**
       * \brief Array of outline points.
       */
     QPointArray outlinepoints;
};

/** \class CanvasCutEllipse
  * \brief A descendent of QCanvasPolygon for ellipses with arcs missing, ie pie slices.
  * @author Ryan Rusaw
  */
class CanvasCutEllipse : public QCanvasPolygon
{
public:
     /**
      * \brief Constructor.
      * @param w width of bounding rectangle
      * @param h height of bounding rectangle
      * @param startangle angle to start external arc at
      * @param arc angular arc length of external arc.
      * @param canvas canvas containing for item
      */
    CanvasCutEllipse(int w,int h, int startangle, int arc, QCanvas *canvas);
     /**
       * \brief Deconstructor
       */
     ~CanvasCutEllipse() { this->hide(); };
     /**
       * \brief Runtime typing value
       * @return Runtime typing value
       */
     int rtti() const;
     /**
       * \brief Gets internal outline points array
       * @return outlinepoints
       */
     QPointArray outlinePoints() { return outlinepoints; };
private:
     /**
       * \brief Array of outline points.
       */
     QPointArray outlinepoints;
};

#endif
