/***************************************************************************
                          notesgen.cpp  -  description
                             -------------------
    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "notesgen.h"
#include "putils.h"

NotesGen::NotesGen()
{
     flag = false;
     itemdepth = 0;
     olddepth = 0;
     ol = false;
}

NotesGen::~NotesGen()
{
}

QString NotesGen::getBullet(int depth, bool usecounter)
{
     QString returnstring;
     for ( int i = 0; i < depth ; i++)
     {
          returnstring.append("&nbsp;&nbsp;&nbsp;&nbsp;");
     }
     returnstring.remove(0,24);
     if (!usecounter)
     { 
          returnstring.append("-  ");
     }
     else
     {
          QString q = "";
          q.setNum(depth);
          cnode = cnode->findChildWithNameAndValue("text:list-level-style-number","text:level", q);
          if (cnode != NULL)
          {
               QString lnp = findStyleProperty(cnode,"style:num-prefix");
               QString nf = findStyleProperty(cnode,"style:num-format");
               if ( nf == "a")
               {
                    lnp += (char)(depthcounter[depth-1]+'a'-1);
               }
               else if ( nf == "A")
               {
                    lnp += (char)(depthcounter[depth-1]+'A'-1);
               }
               else if ( nf == "1")
               {
                    QString w = "";
                    w.setNum(depthcounter[depth-1]);
                    lnp += w;
               }
               else if ( nf == "i")
               {
                    lnp += pUtils::roman(depthcounter[depth-1],false);
               }
               else if ( nf == "I")
               {
                    lnp += pUtils::roman(depthcounter[depth-1],true);
               }
               lnp += findStyleProperty(cnode,"style:num-suffix");
               returnstring.append(lnp);
          }
     } 
     return(returnstring);
}

QString NotesGen::recursiveGen(pXmlNode *node)
{
     QString text, brk, pre, post, bullet, sz, algn, fnt;
     char size[5];
     char font[256];
     brk="";
     QListIterator<pXmlNode> it(node->children);
     for ( ; it.current(); ++it)
     {
         if (flag)
         {
              brk = "<BR>";
         }
         pXmlNode *c = it.current();
         if (c->getNodeName() == "text:p")
         {
              if ( c->getText().length() > 0)
              {
                   // Get font alignment
                   algn = findStylePropertyWithParentDelimiter(c,"draw:page","fo:text-align");
                   if ( algn == "center")
                   {
                       pre.append("<CENTER>");
                       post.prepend("</CENTER>");
                   }
                   // Get point size of font
                   sz = findStylePropertyWithParentDelimiter(c,"draw:page","fo:font-size");
                   int s = (int)sz.toFloat();
                   if ( s <= 10)
                   {
                      sprintf(size,"-1");
                   }
                   else if ( s <= 15)
                   {
                      sprintf(size,"");
                   }
                   else if ( s <= 20)
                   {
                      sprintf(size,"+1");
                   }
                   else if ( s <= 30)
                   {
                      sprintf(size,"+2");
                   }
                   else if ( s <= 40)
                   {
                      sprintf(size,"+3");
                   }
                   else if ( s <= 50)
                   {
                      sprintf(size,"+4");
                   }
                   else
                   {
                      sprintf(size,"+5");
                   }
                   // Get family of font
                   fnt = findStylePropertyWithParentDelimiter(c,"draw:page","fo:font-family");
                   if ( strlen(size) > 0)
                   {
                        sprintf(font,"<FONT size=%s face=%s>",size,fnt.latin1());
                        QString temp(font);
                        pre.append(temp);
                   }
                   else
                   {
                        sprintf(font,"<FONT face=%s>",fnt.latin1());
                        QString temp(font);
                        pre.append("<FONT face="+ fnt +">");
                   }
                   post.prepend("</FONT>");
                   if ( findStylePropertyWithParentDelimiter(c,"draw:text-box","fo:font-style").length() > 0)
                   {
                       pre.append("<I>");
                       post.prepend("</I>"); 
                   }
                   if ( findStylePropertyWithParentDelimiter(c,"draw:text-box","fo:font-weight").length() > 0)
                   {
                       pre.append("<B>");
                       post.prepend("</B>");
                   }
                   if ( findStylePropertyWithParentDelimiter(c,"draw:text-box","style:text-underline").length() > 0)
                   {
                       pre.append("<U>");
                       post.prepend("</U>");
                   }
                   if (findStylePropertyWithParentDelimiter(c,"draw:page","text:enable-numbering").length() > 0)
                   {
                        if (olddepth != itemdepth)
                        {
                             if (olddepth > itemdepth)
                                   depthcounter[itemdepth-1] = 0;
                             else
                                   depthcounter[itemdepth-1] = 1;
                             olddepth = itemdepth;
                        }
                        else
                             depthcounter[itemdepth-1]++;
                        if ( findStylePropertyWithParentDelimiter(c,"draw:page","text:enable-numbering") != "false")
                        {
                             bullet = getBullet(itemdepth, ol);
                        }
                   }
                   else
                   {
                        bullet = getBullet(itemdepth, ol);
                   }
                   text.append(brk + "<NOBR>" + bullet + pre +  c->getText() + post + "</NOBR>");
                   flag = true; 
                   QString tmp(recursiveGen(c));
                   text.append(tmp);                   
              }
              else
              {
                   text.append(brk);
                   flag = true; 
                   text.append(recursiveGen(c));
              }
         }
         else if (c->getNodeName() == "text:unordered-list")
         {
              if ( c->getA().value("text:style-name").length() > 0)
              {
                   cnode = content->findChildWithNameAndValue("text:list-style","style:name",c->getA().value("text:style-name"));
              }
              bool tempbool = ol;
              ol = false;
              text.append(recursiveGen(c));
              ol = tempbool;
              continue;
         }
         else if (c->getNodeName() == "text:ordered-list")
         {
              if ( c->getA().value("text:style-name").length() > 0)
              {
                   cnode = content->findChildWithNameAndValue("text:list-style","style:name",c->getA().value("text:style-name"));
              }
              bool tempbool = ol;
              ol = true;
              text.append(recursiveGen(c));
              ol = tempbool;
              continue;
         }
         else if (c->getNodeName() == "text:list-item")
         {
              itemdepth++;
              text.append(recursiveGen(c));
              itemdepth--;
              continue;
         }
         else if (c->getNodeName() == "text:span" )
         {    
              if ( c->getText().length() > 0)
              {
                   // Get font alignment
                   algn = findStylePropertyWithParentDelimiter(c,"draw:page","fo:text-align");
                   if ( algn == "center")
                   {
                       pre.append("<CENTER>");
                       post.prepend("</CENTER>"); 
                   }
                   // Get point size of font
                   sz = findStylePropertyWithParentDelimiter(c,"draw:page","fo:font-size");
                   int s = (int)sz.toFloat();
                   if ( sz.toFloat() <= 10)
                   {
                      sprintf(size,"-1");
                   }
                   else if ( s <= 15)
                   {
                      sprintf(size,"");
                   }                 
                   else if ( s <= 20)
                   {
                      sprintf(size,"+1");
                   }
                   else if ( s <= 30)
                   {
                      sprintf(size,"+2");
                   }
                   else if ( s <= 40)
                   {
                      sprintf(size,"+3");
                   }
                   else if ( s <= 50)
                   {
                      sprintf(size,"+4");
                   }
                   else
                   {
                      sprintf(size,"+5");
                   }                  
                   // Get family of font
                   fnt = findStylePropertyWithParentDelimiter(c,"draw:page","fo:font-family");
                   if ( strlen(size) > 0)
                   {
                        sprintf(font,"<FONT size=%s face=%s>",size,fnt.latin1());
                        QString temp(font);
                        pre.append(temp);
                   }
                   else
                   {
                        sprintf(font,"<FONT face=%s>",fnt.latin1());
                        QString temp(font);
                        pre.append(temp);
                   }
                   post.prepend("</FONT>");
                   
                   if ( findStylePropertyWithParentDelimiter(c,"draw:text-box","fo:font-style").length() > 0)
                   {
                       if ( findStylePropertyWithParentDelimiter(c,"draw:text-box","fo:font-style") != "normal")
                       {
                            pre.append("<I>");
                            post.prepend("</I>");
                       }
                   }
                   if ( findStylePropertyWithParentDelimiter(c,"draw:text-box","fo:font-weight").length() > 0)
                   {
                       if (findStylePropertyWithParentDelimiter(c,"draw:text-box","fo:font-weight") != "normal")
                       {
                            pre.append("<B>");
                            post.prepend("</B>");
                       }
                   }
                   if ( findStylePropertyWithParentDelimiter(c,"draw:text-box","style:text-underline").length() > 0)
                   {
                       if (findStylePropertyWithParentDelimiter(c,"draw:text-box","style:text-underline") != "none")
                       {
                            pre.append("<U>");
                            post.prepend("</U>");
                       }
                   }
                   QString temptext = "<NOBR>"; temptext += pre; temptext += c->getText();
                   temptext += post; temptext += "<NOBR>";
                   text.append(temptext);
                   continue;
              }
              else if ( c->children.count() > 0)
              {
                   if ( c->children.first()->getNodeName() == "text:line-break")
                   {
                        text.append(brk);
                        continue;
                   }
              }
              else
              {
                   continue;
              }
         }
         else
         {
              continue;
         }
     }
     return(text);
}

QString NotesGen::generateText(PresentationBrowser* pb, int i)
{
     content = &(pb->handler->content);
     style = &(pb->handler->style);
     pXmlNode *page = content->findFirstChildWithName("office:body")->children.at(i-1);
     pXmlNode *notes = page->findFirstChildWithName("presentation:notes");
     pXmlNode *tb = notes->findFirstChildWithName("draw:text-box");
     return(recursiveGen(tb));
}

QString NotesGen::findStyleProperty(pXmlNode *node, const QString &val)
{
     QString rstring = node->getA().value(val);
     pXmlNode *current;
     if (rstring.length() > 0)
     {
          return rstring;
     }
     // search the styles
     if ( node->getA().value("draw:style-name").length() > 0 )                   // graphic objects
          current = content->findChildWithNameAndValue("style:style","style:name",node->getA().value("draw:style-name"), TRUE);
     else if ( node->getA().value("presentation:style-name").length() > 0)       // text boxes
          current = content->findChildWithNameAndValue("style:style","style:name",node->getA().value("presentation:style-name"), TRUE);
     else                                                                        // actual text has a style too
          current = content->findChildWithNameAndValue("style:style","style:name",node->getA().value("text:style-name"), TRUE);
     if (current != NULL)
     {
          if (!current->children.isEmpty())
          {
               rstring = current->children.first()->getA().value(val);
               if (rstring.length() > 0)
               {
                      
                      return rstring;
               }
          }
          rstring = current->getA().value(val);
          if (rstring.length() > 0)
          {
               
               return rstring;
          }
     }
     else
     {
          return "";
     }
     // search all parent styles now
     while (current->getA().value("style:parent-style-name").length() > 0 )
     {
           current = style->findChildWithNameAndValue("style:style","style:name",current->getA().value("style:parent-style-name"));
           if (current != NULL)
           {
               if (!current->children.isEmpty())
               {
                    rstring = current->children.first()->getA().value(val);
                    if (rstring.length() > 0)
                    {
                           return rstring;
                    }
               }
               rstring = current->getA().value(val);
               if (rstring.length() > 0)
               {
                      return rstring;
               }
           }
           else
           {
               continue;
           }
     }
     // finally search the page master style
     current = style->findChildWithNameAndValue("style:master-page","style:master-page-name",node->getA().value("draw:master-style-name"));
     current = style->findChildWithNameAndValue("style:style","style:name",current->getA().value("draw:style-name"));
     if (current != NULL)
     {
         if (!current->children.isEmpty())
         {
              rstring = current->children.first()->getA().value(val);
              if (rstring.length() > 0)
              {
                     return rstring;
              }
         }
         rstring = current->getA().value(val);
         if (rstring.length() > 0)
         {
                return rstring;
         }
     }
     // did not find it.
     return "";
}


/* A shorter alias for findStylePropertyInStyleRecursive() */

QString NotesGen::findStylePropertyInStyleRecursive(pXmlNode *node, const QString &val)
{
     QString k = findStyleProperty(node, val);
     return k;
}

QString NotesGen::findStylePropertyWithParentDelimiter(pXmlNode *node, const QString &delimiter, const QString &val)
{
     QString rstring = findStyleProperty(node,val);
     if (rstring.length() > 0)
         return rstring;
     else
     {
          pXmlNode *p = node->getParent();
          while ( p->getNodeName() != delimiter)
          {
               rstring = findStyleProperty(p,val);
               if ( rstring.length() > 0)
               {
                    return rstring;
               }
               else
               p = p->getParent();
          }
     }
     return "";
}
