/***************************************************************************
                          notesgen.h  -  description
                             -------------------
    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NOTESGEN_H
#define NOTESGEN_H

#include <qobject.h>
#include "pbrowser.h"
#include "pcontent.h"
#include "putils.h"

#define MAX_DEPTH 10

/** \class NotesGen
  * \brief A class for the generating HTML text of the presentation notes.
  * @author Ryan Rusaw
  */
class NotesGen : public QObject
{
public:
    /**
      * \brief Constructor
      */ 
     NotesGen();
    /**
      * \brief Deconstructor
      */
     ~NotesGen();
    /**
      * \brief generate the HTML to be displayed.
      * @param pb Pointer to the PresentationBrowser object that contains the content and style trees
      * @param i Page to generate notes for
      * @return A string that contains the HTML text.
      */
     QString generateText(PresentationBrowser* pb, int i);
private:
    /**
      * \brief generate the HTML to be displayed. Internal use only. 
      */
     QString recursiveGen(pXmlNode *node);
    /**
      * \brief Deprecated alias for findStyleProperty.
      * @see findstylePropertyfindStyleProperty(pXmlNode *node,const QString &val);
      */
     QString findStylePropertyInStyleRecursive(pXmlNode *node,const QString &val);
    /**
      * \brief Finds style property for node.
      * Searches logically through defined nodes and back to master-page-style.
      * @param node node to start search in
      * @param val Property to search for
      * @return value of property, or empty zero length string if not found.
      */
     QString findStyleProperty(pXmlNode *node, const QString &val);
    /**
      * \brief Finds style property for node
      * Searches back up pXmlNode tree till it encounters a node named delimiter. Used in text rendering.
      * @param node node to start search in
      * @param val Property to search for
      * @param delimiter delimiter value
      * @return value of property, or empty zero length string if not found.
      */
     QString findStylePropertyWithParentDelimiter(pXmlNode *node,const QString &val,const QString &delimiter);
    /**
      * \brief Generates HTML for the bullets of ordered lists.
      */
     QString getBullet(int depth, bool usecounter);  
    /**
      * \brief Internal rendering flag
      */
     bool flag;
    /**
      * \brief Pointer to content root.
      */
     pXmlNode *content;
    /**
      * \brief Pointer to style root.
      */
     pXmlNode *style;
    /**
      * \brief Depth of current item in tree.
      */
     int itemdepth;
    /**
      * \brief Depth of old item in tree
      */
     int olddepth;
    /**
      * \brief Ordered list = true
      */
     bool ol;
    /**
      * \brief Pointer to node. Internal use only.
      */
     pXmlNode *cnode;
    /**
      * \brief Array of item depths
      */
     int depthcounter[MAX_DEPTH];
};

#endif
