/***************************************************************************
                          pbrowser.cpp  -  description
                             -------------------
    begin                : Thu Jul 11 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *

 ***************************************************************************/

#include <qwidget.h>
#include <qcanvas.h>
#include <qmainwindow.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpoint.h>
#include <qstack.h>
#include <qvector.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qpe/process.h>
#include "pbrowser.h"
#include "pparser.h"
#include <qpe/applnk.h>
#include <qpe/resource.h>
#include <qpe/qpeapplication.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include "renderitems.h"
#include "render3d.h"
#include "rendertxt.h"
#include "renderrect.h"
#include "renderbg.h"
#include "renderellipse.h"
#include "renderimage.h"
#include "renderole.h"
#include "pzipextractor.h"

#define FACTOR_1X         1
#define FACTOR_2X         2
#define FACTOR_3X         3
#define FACTOR_FULLSCREEN 4

// Presentation Browser
void PresentationBrowser::contentsMousePressEvent(QMouseEvent * e)
{
     fullscreen = false;
     enableFullscreen();
}

PresentationBrowser::PresentationBrowser(QCanvas *c, QWidget *w, const char *name, WFlags fl)
    : QCanvasView(c, w, name, fl)
{
    setFactor(1); // base conversion factor to fit on 240*320 screen
}

void PresentationBrowser::renderPage(int i)
{
    QTime start;
    QString etime;
    clearPage();
    thispage = i;
                                
    pXmlNode * page = handler->content.findFirstChildWithName("office:body")->children.at(i-1);
    emit renderProgressStart( page->children.count());
    int j = 1;
    RenderBG *r2 = new RenderBG(this, canvas(), &(handler->content), &(handler->style), &(handler->images));
    r2->setFactor(factor);
    r2->setReplace(replaceitems);
    start = QTime::currentTime();
    r2->render(page);
    etime.setNum(start.elapsed());
    qWarning( "Elapsed BG Time = " + etime);
    emit renderProgressUpdate(j);
    j++;
    
    RenderItems *renderer = new RenderItems(this, canvas(), &(handler->content), &(handler->style), &(handler->images));
    renderer->setFactor(factor);
    renderer->setReplace(replaceitems);
    QListIterator<pXmlNode> it(page->children);
    for(;it.current();++it)
    {
        pXmlNode *item = it.current();
        QString nn = item->getNodeName();
        start = QTime::currentTime();
        if (nn == "draw:rect")
        {
             ((RenderRect*)renderer)->render(item);
        }
        else if ((nn == "draw:ellipse") || (nn == "draw:circle"))
        {
             ((RenderEllipse*)renderer)->render(item);
        }
        else if (nn == "draw:text-box")
        {
             ((RenderTxt*)renderer)->render(item);
        }
        else if (nn == "draw:image")
        {
             ((RenderImage*)renderer)->render(item);
        }
        else if (nn == "dr3d:scene")
        {
             ((Render3D*)renderer)->render(item);
        }
        else if (nn == "draw:object")
        {
             ((RenderOLE*)renderer)->render(item, &objectfilelist, &objectnamelist);
        }
        else
        {
             break;
        }
        etime.setNum(start.elapsed());
        qWarning( "Elapsed Time for " +nn+ " = " + etime);
        emit renderProgressUpdate(j);
        j++;
    }
    if (!fullscreen)
         canvas()->update();
}

void PresentationBrowser::parseFiles(const DocLnk &lnkDoc)
{
    pZipExtractor *pz = new pZipExtractor();
    //unzFile uf = unzOpen(lnkDoc.file().latin1());
    QByteArray *ba = NULL;
    if ( pz->openFile(lnkDoc) == 0 )
    {
        QMessageBox::critical(this, tr("Error"), tr("File cannot be unzipped!"));
        return;
    }
    //if ((ba = extractFileToBuffer(uf,(char*)"META-INF/manifest.xml")) == NULL)
    QString tempstr("META-INF/manifest.xml");
    if ((ba = pz->extractFile(tempstr))== NULL)
    {
        QMessageBox::critical(this, tr("Error"), tr("Extracting manifest.xml failed!"));
        return;
    }
    QTextStream ts( *ba, IO_ReadWrite );
    QXmlInputSource source( ts );
    QXmlSimpleReader reader;
    handler = new PParser();
    reader.setContentHandler( handler );
    reader.parse( source );
    sxifile *ptr = new sxifile();
    ptr = handler->file.first();
    while (ptr != NULL)
    {
        if (ptr->mediatype == "text/xml") // Presentation files.
        {
            if ((ba = pz->extractFile(ptr->fullpath)) == NULL)
            {
                QMessageBox::critical(this, tr("Error"), tr("Extracting file failed!"));
                return;
            }
            QTextStream ts_xml( *ba, IO_ReadWrite );
            if ( ptr->fullpath.contains("/") ) // OLE Objects are not in the root dir.
            {
                objectfilelist.append(ba);
                objectnamelist.append(new QString(ptr->fullpath));
            }
            QXmlInputSource source_xml( ts_xml );
            reader.setContentHandler( handler );
            reader.parse( source_xml );
        }
        if (ptr->mediatype.contains("image/",FALSE)) // images
        {
            if ((ba = pz->extractFile(ptr->fullpath)) == NULL)
            {
                QMessageBox::critical(this, tr("Error"), tr("Extracting file failed!"));
                return;
            }
            pImage *i = new pImage();
            QImage *img = new QImage();
            if ( img->loadFromData(*ba) )
            {
                 i->image = img;
            }
            i->filename = ptr->fullpath;
            if (handler->images.contains(i) == 0)
            {
              handler->images.append(i);
            }
        }
        if (ptr->mediatype.contains("application/vnd.sun.xml",FALSE)) // directory entries
        {
            // Do nothing, we dont really care about these.
        }
        if (ptr->mediatype.length() == 0) // OLE files sometimes are untyped.
        {
            if ((ba = pz->extractFile(ptr->fullpath)) == NULL)
            {
                 QMessageBox::critical(this, tr("Error"), tr("Extracting file failed!"));
                 return;
            }
            objectfilelist.append(ba);
            objectnamelist.append(new QString(ptr->fullpath));
        }       
        ptr = handler->file.next();
    }
    pz->closeFile();
}

int PresentationBrowser::pageCount()
{
    pXmlNode * temp = handler->content.findFirstChildWithName("office:body");
    return temp->children.count();
}

void PresentationBrowser::clearPage()
{
   QCanvasItemList cl = canvas()->allItems();
   QCanvasItemList::Iterator it;
   for( it = cl.begin(); it != cl.end(); ++it )
   {
          (*it)->hide();
   }
   RenderItems *renderer = new RenderItems(this, canvas(), &(handler->content), &(handler->style), &(handler->images));
   if (renderer != NULL )
   {
     renderer->clearLayers();
     renderer->setReplace(replaceitems);
   }
   QPixmap *pm = new QPixmap();
   canvas()->setBackgroundPixmap(*pm);
   QColor *color = new QColor(255,255,255);
   canvas()->setBackgroundColor(*color);
}

void PresentationBrowser::setFactor(int f)
{
  switch(f)
  {
  case FACTOR_1X:
     factor = 0.945;        // sized to fit
     break;
  case FACTOR_2X:
     factor = 0.4725;       // 2x
     break;
  case FACTOR_3X:
     factor = 0.315;        // 3x
     break;
  case FACTOR_FULLSCREEN:
     factor = 0.690;         // fullscreen = 1.5x rotated with no border.
     break;
  default:
     factor = factor;       // if invalid magnification do nothing
     break;
  }
}

void PresentationBrowser::setReplace(bool r)
{
     replaceitems = r;
     RenderItems *renderer = new RenderItems(this, canvas(), &(handler->content), &(handler->style), &(handler->images));
     if (renderer != NULL) renderer->setReplace(r);
}

void PresentationBrowser::resizeEvent(QResizeEvent *e){
    if(size() != QPEApplication::desktop()->size())
    {
         if (fullscreen)
                enableFullscreen();
    }
}

void PresentationBrowser::focusInEvent(QFocusEvent *e)
{
    //if (fullscreen)
    //   enableFullscreen();
}


void PresentationBrowser::enableFullscreen()
{
     if (fullscreen)
     {
         setFixedSize(QPEApplication::desktop()->size());
         showNormal();
         reparent(0, WStyle_Customize | WStyle_NoBorder, QPoint(0,0));
         showFullScreen();
     }
     else
     {
         setFixedSize(235,180);
         emit offFullscreen();
     }
}

void PresentationBrowser::setFullscreen(bool b)
{
     fullscreen = b;
}

void PresentationBrowser::keyPressEvent(QKeyEvent *e)
{
     if(fullscreen)
     {
          emit keyPressedInFullscreenEvent(e);
     }
     e->ignore();
}

void PresentationBrowser::showFullscreenPixmap()
{
    QPixmap *pm = new QPixmap(canvas()->size());
    resizeContents(canvas()->width(),canvas()->height());
    QPainter *p = new QPainter(pm);
    drawContents(p,0,0,canvas()->width(),canvas()->height());
    enableFullscreen();
    if (canvas()->width() != QPEApplication::desktop()->width())
    {
        canvas()->resize(QPEApplication::desktop()->width(),QPEApplication::desktop()->height());
    }
    clearPage();
    QCanvasSprite *s;
    QList<QPixmap> pml;
    QList<QPoint> ptl;
    ptl.append(new QPoint(0,0));
    QWMatrix m;
    m.rotate(-90);
    QPixmap *pm_rotated = new QPixmap(pm->xForm(m));
    pml.append(pm_rotated);
    QCanvasPixmapArray *pma1 = new QCanvasPixmapArray(pml, ptl);
    s = new QCanvasSprite(pma1,canvas());
    s->show();
    pml.remove(pm);
}
