/***************************************************************************
                          pbrowser.h  -  description
                             -------------------
    begin                : Thu Jul 11 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef PBROWSER_H
#define PBROWSER_H

#include <qwidget.h>
#include <qcanvas.h>
#include <pparser.h>
#include <qpe/process.h>
#include <qpe/applnk.h>
#include "unzip.h"
#include "renderitems.h"

#define CM_TO_PXL 8
#define SCALE_FACTOR 4.75

class RenderItems;

/** \class PresentationBrowser
  * \brief QCanvasView descendent that handles parsing and rendering of document.
  * @author Ryan Rusaw
  */
class PresentationBrowser : public QCanvasView
{
    Q_OBJECT

public:
    /**
      * \brief Constructor
      * @param c canvas
      * @param w parent widget
      * @param name name of PresentationBrowser
      * @param fl widget flags
      */
    PresentationBrowser (QCanvas *c, QWidget *w, const char *name, WFlags fl = 0);
    /**
      * \brief Deconstructor
      */
    ~PresentationBrowser()  { };
    /**
      * \brief Parses files into into pXmlNode tree's.
      * @param lnkDoc DocLnk to .sxi file to parse
      */
    void parseFiles(const DocLnk &lnkDoc);
    /**
      * \brief The page count of the current document
      * @return the # of pages
      */
    int pageCount();
    /**
      * \brief renders the given page to canvas
      * @param i page number to render
      */
    void renderPage(int i);
    /**
      * \brief clears the canvas
      */
    void clearPage();
    /**
      * \brief Sets the scale factor
      *   0 = Scale factor 1x
      *   1 = Scale factor 2x
      *   2 = Scale factor 3x
      *   3 = Scale factor full page
      * @param f scale factor
      */
    void setFactor(int f);
    /**
      * \brief Sets the boolean replace objects flag.
      * @param r value to set flag to 
      */
    void setReplace(bool r);
    /**
      * \brief XML handler for document
      */
    PParser *handler;
    /**
      * \brief list of subobject documents files.
      */
    QList<QByteArray> objectfilelist;
    /**
      * \brief list of all subobject documents names.
      */
    QList<QString> objectnamelist;
    /**
      * \brief Event handler for resize events
      * @param e Event
      */  
    void resizeEvent(QResizeEvent *e);
    /**
      * \brief Event handler for focus in events
      * @param e Event
      */
    void focusInEvent(QFocusEvent *e);
    /**
      * \brief Enables fullscreen mode 
      */
    void enableFullscreen();
    /**
      * \brief Sets internal fullsize mode flag
      */
    void setFullscreen(bool);
    /**
      * \brief Displays Canvas to fullscreen as a pixmap
      */
    void showFullscreenPixmap();

private:
    /**
      * \brief internal page counter
      */
    int pages;
    /**
      * \brief Internal rendering factor.
      */
    float factor;
    /**
      * \brief current page number
      */
    int thispage;
    /**
      * \brief replace items flag
      */
    bool replaceitems;
    /**
      * \brief fullscreen mode flag
      */
    bool fullscreen;
    /**
      * \brief Event handler for keypress events
      * @param e event
      */
    void keyPressEvent(QKeyEvent *e);

protected:
    /**
      * \brief Event handler for mouse press events
      * @param e event
      */
    void contentsMousePressEvent(QMouseEvent *e);

signals:
    /**
      * \brief signals when rendering starts
      */
    void renderProgressStart(int);
    /**
      * \brief signals for internal rendering step
      */
    void renderProgressUpdate(int);
    /**
      * \brief signals when changing from fullscreen mode
      */
    void offFullscreen();
    /**
      * \brief signals when keypressed in fullscreen mode
      */
    void keyPressedInFullscreenEvent(QKeyEvent *);  
};

#endif //PBROWSER_H
