/***************************************************************************
                          pcontent.h  -  description
                             -------------------
    begin                : Fri Jul 12 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef PCONTENT_H
#define PCONTENT_H

#include <qxml.h>
#include <qobject.h>
#include <qlist.h>

/** \class pXmlNode
  * \brief Class that forms basis of xml tree structures.
  * @author Ryan Rusaw
  */

class pXmlNode : public QObject
{

public:
/**
  * \brief Constructor
  * Creates a blank node.
  */
    pXmlNode();
/**
  * \brief Constructor
  * Creates a node with given values.
  * @param p Parent node.
  * @param qName Qualified name of xml node.
  * @param attrib QXmlAttributes of node.
  */
    pXmlNode(pXmlNode *p, const QString &qName, const QXmlAttributes &attrib);
/**
  * \brief Deconstructor
  */
    ~pXmlNode(){};
/**
  * \brief List of children nodes.
  */
    QList<pXmlNode> children;
/**
  * \brief Gets internal nodename
  * @return nodename
  */
    QString &getNodeName();
/**
  * \brief Sets internal nodename
  * @param name Name to set nodename to.
  */
    void setNodeName(const QString &name);
/**
  * \brief Gets internal text value
  * @return text.
  */
    QString &getText();
/**
  * \brief Sets internal text value
  * @param t text value to set.
  */
    void setText(const QString &t);
/**
  * \brief Gets pointer to internal parent pointer.
  * @return parent
  */
    pXmlNode *getParent();
/**
  * \brief Sets internal pointer to parent.
  * @param node pointer to set.
  */
    void setParent(pXmlNode *node);
/**
  * \brief Gets internal QXmlAttributes variable.
  * @return a
  */    
    QXmlAttributes &getA();
/**
  * \brief Sets internal QXmlAttibutes a value.
  * @param attr QXmlAttibutes to set.
  */
    void setA(const QXmlAttributes &attr);
/**
  * \brief Find first child with a given nodename
  * @param s search for this nodename
  * @param recurse recurses up children if true
  * @return Pointer to childnode named s
  */
    pXmlNode *findFirstChildWithName(const QString &s, bool recurse = TRUE);
/**
  * \brief Find last child with a given nodename
  * @param s search for this nodename
  * @param recurse recurses up children if true
  * @return Pointer to childnode named s
  */
    pXmlNode *findLastChildWithName(const QString &s, bool recurse = TRUE);
/**
  * \brief Find first child with a given nodename and a given attribute value
  * @param s search for this nodename
  * @param qName Qualified name of attribute
  * @param val value of attrubute
  * @param recurse recurses up children if true
  * @return Pointer to childnode named s with attibute qName = val.
  */
    pXmlNode *findChildWithNameAndValue(const QString &s, const QString &qName, const QString &val, bool recurse = TRUE);

private:
/**
  * \brief Pointer to text
  */
    QString *text;
/**
  * \brief Pointer to nodename
  */
    QString *nodename;
/**
  * \brief Pointer to parent node
  */
    pXmlNode *parent;
/**
  * \brief Xml Attributes of node.
  */
    QXmlAttributes a;

};

/** \class pImage
  * \brief Image class that holds raw image data and filename.
  * @author Ryan Rusaw
  */

class pImage : public QObject
{

public:
/**
  * \brief Constructor
  */
    pImage();
/**
  * \brief Deconstructor
  */    
    ~pImage(){};
/**
  * \brief Filename of image in .sxi archive ie. Pictures/Test.png
  */
    QString filename;
/**
  * \brief Pointer to QImage of image.
  */
    QImage *image;
};

#endif



