/***************************************************************************
                          poqet.cpp  -  description
                             -------------------
    begin                : Thu Jul 11 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *

 ***************************************************************************/
#include "poqet.h"

#include <qvariant.h>
#include <qslider.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qtextview.h>
#include <qlabel.h>
#include <qprogressbar.h>
#include <qpe/fileselector.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qcanvas.h>
#include <qstack.h>
#include <qpe/resource.h>
#include <qpe/fontdatabase.h>
#include <qtextbrowser.h>
#include "pbrowser.h"
#include "pparser.h"

/* 
 *  Constructs a PoqetPresenter which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 */

PoqetPresenter::PoqetPresenter( QWidget * parent,  const char* name, WFlags fl )
    : QMainWindow( parent, name, fl )
{
    if ( !name )
	  setName( "PoqetPresenter" );
    resize( 240, 314 );
    setCaption( tr( "PoqetPresenter" ) );
    cw = new QWidget( this, "qt_central_widget");
    setCentralWidget( cw );
    // widgets
    Widgets();
    // actions
    Actions();
    // menubar
    Menus();
    // toolbars
    Toolbars();
    // signals and slots connections
    SigsNSlots();
    // initialize static utility class
    pUtils *utils = new pUtils();
}

/*  
 *  Destroys the object and frees any allocated resources
 */
PoqetPresenter::~PoqetPresenter()
{
    // no need to delete child widgets, Qt does it all for us
}

void PoqetPresenter::Menus()
{
    toolbar = new QPEToolBar( this );
    menubar = new QPEMenuBar( toolbar );
    fileMenu = new QPopupMenu( this );
    fileOpenAction->addTo( fileMenu );
    fileMenu->insertSeparator();
    fileMenu->insertSeparator();
    fileExitAction->addTo( fileMenu );
    menubar->insertItem( tr( "&File" ), fileMenu );
    pageMenu=new QPopupMenu(this);
    prevPageAction->addTo(pageMenu);
    nextPageAction->addTo(pageMenu);
    menubar->insertItem( tr( "&Slide" ), pageMenu );
    helpMenu = new QPopupMenu( this );
    helpMenu->insertSeparator();
    helpAboutAction->addTo( helpMenu );
    menubar->insertItem( tr( "&Help" ), helpMenu );
}

void PoqetPresenter::Toolbars()
{
    toolbar=new QPEToolBar(this);
    setRightJustification(TRUE);
    setToolBarsMovable( FALSE );
    
    // add tool buttons to the toolbar after the menus
    fileOpenAction->addTo(toolbar);
    prevPageAction->addTo(toolbar);
    nextPageAction->addTo(toolbar);
    helpAboutAction->addTo(toolbar);
}

void PoqetPresenter::Widgets()
{
    fileSelector = new FileSelector("application/vnd.sun.xml.impress", this, QString::null);
    fileSelector->setNewVisible(FALSE);
    
    TabW = new QTabWidget(centralWidget(), "tabwidget");
    TabW->setGeometry(QRect(0,0,240,260));
    TabW->setTabPosition(QTabWidget::Bottom);

    TabPB = new QWidget(centralWidget(), "tabpb");
    TabPB->setGeometry( QRect( 0, 0, 240, 180 ) );
    TabMeta = new QWidget(centralWidget(), "tabmeta");
    TabMeta->setGeometry( QRect( 0, 0, 240, 180 ) );
    TabNotes = new QWidget(centralWidget(), "tabnotes");
    TabNotes->setGeometry( QRect( 0, 0, 240, 180 ) );

    Mag = new MenuButton(TabPB,"magbutton");
    Mag->setGeometry( QRect( 2, 208, 118, 20) );
    QIconSet *temp = new QIconSet(Resource::loadPixmap("find"));
    Mag->insertItem ( *temp , "1x magnification" );
    Mag->insertItem ( *temp , "2x magnification" );
    Mag->insertItem ( *temp , "3x magnification" );
    Mag->insertItem ( *temp , "Fullscreen Mode" );

    Progress = new QProgressBar( TabPB );
    Progress->setGeometry( QRect( 124, 208, 110, 20) );
    
    PageLabel = new QLabel(tr("Slide 0/0"),centralWidget(),"pagelabel");
    PageLabel->setGeometry( QRect( 160, 238, 60, 20));

    PresentationBrowser3 = new PresentationBrowser( &canvas, TabPB, "PresentationBrowser3");
    PresentationBrowser3->setGeometry( QRect( 0, 0, 235, 180 ) );
    PresentationBrowser3->setVScrollBarMode(QScrollView::Auto);   
    PresentationBrowser3->setHScrollBarMode(QScrollView::Auto);

    Slider1 = new QSlider( TabPB, "Slider1" );
    Slider1->setGeometry( QRect( 0, 182, 235, 20 ) );
    Slider1->setOrientation( QSlider::Horizontal );
    Slider1->setRange(1,1);
    Slider1->setValue(1);
    Slider1->setLineStep(1);
    Slider1->setTickmarks( QSlider::Both );
    Slider1->setTickInterval(1);

    MetaView = new QListView( TabMeta, "listviewmeta" );
    MetaView->setGeometry( QRect( 0,0, 235, 231 ) );
    MetaView->addColumn(tr("Type"));
    MetaView->addColumn(tr("Value"));

    PNotes = new QTextBrowser( TabNotes, "pnotestext" );
    PNotes->setGeometry ( QRect( 0,0, 235, 231 ) );
    PNotes->setVScrollBarMode(QScrollView::Auto);
    PNotes->setHScrollBarMode(QScrollView::Auto);

    TabW->addTab(TabPB, tr("View"));
    TabW->addTab(TabNotes, tr("Notes"));
    TabW->addTab(TabMeta, tr("Meta"));

    LoadingStatusLabel = new QLabel ( centralWidget(), "loadingstatus" );
    LoadingStatusLabel->setText("  Loading file ...  ");
    LoadingStatusLabel->setGeometry(80,80,80,20);
    LoadingStatusLabel->hide();
}

void PoqetPresenter::Actions()
{
    fileOpenAction = new QAction( this, "fileOpenAction" );
    fileOpenAction->setText( tr( "Open" ) );
    fileOpenAction->setMenuText( tr( "&Open..." ) );
    fileOpenAction->setAccel( 4194383 );
    fileOpenAction->setIconSet(Resource::loadPixmap("fileopen"));
    fileExitAction = new QAction( this, "fileExitAction" );
    fileExitAction->setText( tr( "Exit" ) );
    fileExitAction->setMenuText( tr( "E&xit" ) );
    fileExitAction->setAccel( 0 );
    fileExitAction->setIconSet(Resource::loadPixmap("quit_icon"));
    helpAboutAction = new QAction( this, "helpAboutAction" );
    helpAboutAction->setText( tr( "About" ) );
    helpAboutAction->setMenuText( tr( "&About..." ) );
    helpAboutAction->setAccel( 0 );
    helpAboutAction->setIconSet(Resource::loadPixmap("help_icon"));
    nextPageAction = new QAction(this, "next");
    nextPageAction->setText(tr("Next"));
    nextPageAction->setMenuText( tr( "&Next Slide"));
    nextPageAction->setAccel( 0 );
    nextPageAction->setIconSet(Resource::loadPixmap("forward_inactive"));
    prevPageAction = new QAction(this, "prev");
    prevPageAction->setText(tr("Previous"));
    prevPageAction->setMenuText( tr( "&Previous Slide"));
    prevPageAction->setAccel( 0 );
    prevPageAction->setIconSet(Resource::loadPixmap("back_inactive"));

}

void PoqetPresenter::SigsNSlots()
{
    connect( fileSelector, SIGNAL(closeMe()), this, SLOT(selectorHide()));
    connect( fileSelector, SIGNAL(fileSelected(const DocLnk &)), this, SLOT(selectorFileOpen(const DocLnk &)));
    connect( fileOpenAction, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
    connect( fileExitAction, SIGNAL( activated() ), this, SLOT( fileExit() ) );
    connect( helpAboutAction, SIGNAL( activated() ), this, SLOT( helpAbout() ) );
    connect( Slider1, SIGNAL( sliderReleased() ), this, SLOT( updatePage() ) );
    connect( nextPageAction, SIGNAL( activated()), this, SLOT( viewNextPage() ) );
    connect( prevPageAction, SIGNAL( activated()), this, SLOT( viewPrevPage() ) );
    connect( Mag, SIGNAL( selected(int)), this, SLOT( magnifyUpdate(int)));
    connect( PresentationBrowser3, SIGNAL( renderProgressStart(int) ), this, SLOT (startProgressBar(int)) );
    connect( PresentationBrowser3, SIGNAL( renderProgressUpdate(int) ), Progress, SLOT (setProgress(int)) );
    connect( PresentationBrowser3, SIGNAL( offFullscreen() ), this, SLOT (offFullscreenHandler()) );
    connect( PresentationBrowser3, SIGNAL( keyPressedInFullscreenEvent(QKeyEvent*) ), this, SLOT (keyPressEvent(QKeyEvent*)) );
}

void PoqetPresenter::fileOpen()
{
}

void PoqetPresenter::fileExit()
{
}

void PoqetPresenter::helpAbout()
{
}

void PoqetPresenter::selectorFileOpen(const DocLnk &lnkDoc)
{
}

void PoqetPresenter::selectorShow()
{
}

void PoqetPresenter::selectorHide()
{
}

void PoqetPresenter::updatePage(int i)
{
}

void PoqetPresenter::updatePage()
{
}

void PoqetPresenter::viewNextPage()
{
}

void PoqetPresenter::viewPrevPage()
{
}

void PoqetPresenter::setDocument(const QString& fileref)
{
}

void PoqetPresenter::magnifyUpdate(int index)
{
}

void PoqetPresenter::startProgressBar(int count)
{
}

void PoqetPresenter::stopProgressBar()
{
}

void PoqetPresenter::offFullscreenHandler()
{
}

void PoqetPresenter::keyPressEvent(QKeyEvent *e)
{
     e->ignore();
}



