/***************************************************************************
                          poqet.h  -  description
                             -------------------
    begin                : Thu Jul 11 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *

 ***************************************************************************/
#ifndef POQETPRESENTER_H
#define POQETPRESENTER_H

#include <qvariant.h>
#include <qmainwindow.h>
#include <qmessagebox.h>
#include <qtabwidget.h>
#include <qcheckbox.h>
#include <qtextview.h>
#include <qstack.h>
#include <qlistview.h>
#include <qprogressbar.h>
#include <qpe/applnk.h>
#include <qpe/fileselector.h>
#include <qpe/qpemenubar.h>
#include <qpe/menubutton.h>
#include <qpe/qpetoolbar.h>
#include <qcanvas.h>
#include "pcontent.h"

class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QAction;
class QActionGroup;
class QToolBar;
class QPopupMenu;
class PresentationBrowser;
class QSlider;


/** \class PoqetPresenter
  * \brief Main window class
  * @author Ryan Rusaw
  */
class PoqetPresenter : public QMainWindow
{ 
    Q_OBJECT

public:
    /**
      * Constructor
      * @param parent Parent widget
      * @param name widget name
      * @param fl widget flags
      */
    PoqetPresenter( QWidget* parent = 0, const char* name = 0, WFlags fl = WType_TopLevel );
    /**
      * Deconstructor
      */
    ~PoqetPresenter();
    /**
      * \brief Child widget
      */
    PresentationBrowser* PresentationBrowser3;
    /**
      * \brief Child widget
      */
    QTabWidget *TabW;
    /**
      * \brief Child widget
      */
    QWidget *TabPB;
    /**
      * \brief Child widget
      */
    QWidget *TabMeta;
    /**
      * \brief Child widget
      */
    QWidget *TabNotes;
    /**
      * \brief Child widget
      */
    MenuButton *Mag;
    /**
      * \brief Child widget
      */
    QSlider* Slider1;
    /**
      * \brief Child widget
      */
    QLabel* PageLabel;
    /**
      * \brief Child widget
      */
    QListView* MetaView;
    /**
      * \brief Child widget
      */
    QTextView* PNotes;
    /**
      * \brief Child widget
      */
    QPopupMenu *fileMenu;
    /**
      * \brief Child widget
      */
    QPopupMenu *helpMenu;
    /**
      * \brief Child widget
      */
    QPopupMenu *pageMenu;
    /**
      * \brief Child widget
      */
    QProgressBar *Progress;
    /**
      * \brief QAction for opening
      */
    QAction* fileOpenAction;
    /**
      * \brief QAction for exiting
      */
    QAction* fileExitAction;
    /**
      * \brief QAction for about dialog
      */
    QAction* helpAboutAction;
    /**
      * \brief QAction for advancing to next page
      */
    QAction* nextPageAction;
    /**
      * \brief QAction for going to previous page
      */
    QAction* prevPageAction;
    /**
      * \brief Child widget
      */
    DocLnk currentDoc;
    /**
      * \brief Child widget
      */
    QPEMenuBar *menubar;
    /**
      * \brief Child widget
      */
    QPEToolBar *toolbar;
    /**
      * \brief Child widget
      */
    FileSelector *fileSelector;
    /**
      * \brief Child widget for loading status
      */
    QLabel *LoadingStatusLabel;

public slots:
    /**
      * \brief Opens file selector 
      */
    virtual void fileOpen();
    /**
      * \brief Exits
      */
    virtual void fileExit();
    /**
      * \brief Shows about dialog
      */
    virtual void helpAbout();
    /**
      * \brief Shows next page
      */
    virtual void viewNextPage();
    /**
      * \brief Shows previous page
      */
    virtual void viewPrevPage();
    /**
      * \brief Shows file selector
      */
    virtual void selectorShow();
    /**
      * \brief Hides file selector
      */
    virtual void selectorHide();
    /**
      * \brief Updates current page display
      */
    virtual void updatePage();
    /**
      * \brief Updates page
      * @param i page to update
      */
    virtual void updatePage(int i);
    /**
      * \brief Opens file
      * @param lnkDoc document to open
      */
    virtual void selectorFileOpen(const DocLnk &lnkDoc);
    /**
      * \brief Sets internal document reference name
      * @param fileref name
      */
    virtual void setDocument(const QString &fileref);
    /**
      * \brief Changes magnification and updates page
      * @param index size to change to
      */
    virtual void magnifyUpdate(int index);
    /**
      * \brief Sets up progress bar
      * @param count number of progress steps
      */
    virtual void startProgressBar(int count);
    /**
      * \brief Stops progress bar
      */
    virtual void stopProgressBar();
    /**
      * \brief Updates internal flags for non-fullscreen mode
      */
    virtual void offFullscreenHandler();
    /**
      * \brief Handles key presses passed from child widget
      * @param e event 
      */
    virtual void keyPressEvent(QKeyEvent *e);

private:
    /**
      * \brief Sets up menus. Internal use only.
      */
    void Menus();
    /**
      * \brief Sets up actions. Internal use only.
      */
    void Widgets();
    /**
      * \brief Sets up widgets. Internal use only.
      */
    void Actions();
    /**
      * \brief Sets up toolbars. Internal use only.
      */
    void Toolbars();
    /**
      * \brief Sets up signals and slots. Internal use only.
      */
    void SigsNSlots();

protected:
    /**
      * \brief Pointer to central widget
      */
    QWidget *cw;
    /**
      * \brief Pointer to canvas
      */
    QCanvas canvas;

};

#endif // POQETPRESENTER_H

