/***************************************************************************
                          poqetp.cpp  -  description
                             -------------------
    begin                : Thu Jul 11 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qmainwindow.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include <qcanvas.h>
#include <qevent.h>
#include <qtimer.h>
#include <qpe/config.h>
#include <qpe/resource.h>
#include <qpe/qpeapplication.h>
#include <unistd.h>
#include "poqetp.h"
#include "notesgen.h"

 
PoqetP::PoqetP( QWidget* parent,  const char* name, WFlags fl )
    : PoqetPresenter( parent, name, fl )
{
    // other stuff
    magnifyUpdate(Mag->currentItem());
    fileOpen();
    fullscreen = false;
}

PoqetP::~PoqetP()
{
}

void PoqetP::fileOpen()
{
    selectorShow();
}

void PoqetP::fileExit()
{
    exit(0);
    //QPEApplication::exit(0);
}

void PoqetP::helpAbout()
{
    QMessageBox::about(0,tr("PoqetPresenter"),tr("PoqetPresenter is copyright\n"
                         "2002 by Ryan Rusaw, and\n"
                         "1998 by Gilles Vollant (unzip.*)\n"
                         "and is licensed under the GPL"));
}

void PoqetP::selectorFileOpen(const DocLnk &lnkDoc)
{
  selectorHide();
  copyDocLnk(lnkDoc, currentDoc);
  LoadingStatusLabel->show();
  //this is a workaround to get the selector to disappear before the loading process finishes.
  QTimer *loadFileTimer = new QTimer();
  loadFileTimer->singleShot(1,this,SLOT(timerSlot()));
  documentModified=FALSE;
}

void PoqetP::selectorShow()
{
  setCentralWidget(fileSelector);
  fileSelector->show();
  fileSelector->reread();
}

void PoqetP::selectorHide()
{
   fileSelector->hide();
   setCentralWidget( cw );
}

void PoqetP::updatePage(int i)
{
  if (currentDoc.isValid())
  {
     PresentationBrowser3->clearPage();
     NotesGen *ng = new NotesGen();
     PNotes->setText(ng->generateText(PresentationBrowser3,Slider1->value()));
     if (!fullscreen)
     {
          PresentationBrowser3->renderPage(i);
     }
     stopProgressBar();
  }
  setPrevNext();
}

void PoqetP::updatePage()
{
  if (currentDoc.isValid())
  {
     PresentationBrowser3->clearPage();
     NotesGen *ng = new NotesGen();
     PNotes->setText(ng->generateText(PresentationBrowser3,Slider1->value()));
     if (!fullscreen)
     {
          PresentationBrowser3->renderPage(Slider1->value());
     }
     stopProgressBar();
  }
  char label[256];
  sprintf(label,"Slide %d/%d",Slider1->value(),Slider1->maxValue());
  QString t(label);
  PageLabel->setText(t);
  setPrevNext();
  if(fullscreen)
  {
       QTimer *timer = new QTimer( this );
       connect( timer, SIGNAL(timeout()), this, SLOT(bitBltNow()) );
       timer->start( 50, TRUE );
  }
}

void PoqetP::viewNextPage()
{
  if (Slider1->value() < Slider1->maxValue())
  {
     Slider1->addLine();
     updatePage();
  }
}

void PoqetP::viewPrevPage()
{
  if (Slider1->value() > Slider1->minValue())
  {
     Slider1->subtractLine();
     updatePage();
  }
}

void PoqetP::copyDocLnk(const DocLnk &source, DocLnk &target)
{
  target.setName(source.name());
  target.setFile(source.file());
  target.setLinkFile(source.linkFile());
  target.setComment(source.comment());
  target.setType(source.type());
}

void PoqetP::documentOpen(const QString &str)
{
    DocLnk nf;
    nf.setType("application/vnd.sun.xml.impress");
    nf.setFile(str);
    documentOpen(nf);
}

void PoqetP::documentOpen(const DocLnk &lnkDoc)
{
  if (!lnkDoc.isValid())
  {
    QMessageBox::critical(this, tr("Error"), tr("File cannot be opened!"));
    selectorHide();
    return;
  }
  currentDoc = lnkDoc;
  parseXMLFiles(lnkDoc);
  setPrevNext();
  return;
}

void PoqetP::setDocument(const QString& fileref)
{
    selectorHide();
    documentOpen(DocLnk(fileref));
}

void PoqetP::parseXMLFiles(const DocLnk &lnkDoc)
{
    if (!lnkDoc.isValid())
    {
       QMessageBox::critical(this, tr("Error"), tr("File cannot be opened!"));
       LoadingStatusLabel->hide();
       return;
    }
    PresentationBrowser3->parseFiles(lnkDoc);
    LoadingStatusLabel->hide();
    Slider1->setRange(1,PresentationBrowser3->pageCount());
    Slider1->setTickInterval(1);
    Slider1->setValue(1);
    char label[256];
    sprintf(label,"%d/%d",Slider1->value(),Slider1->maxValue());
    QString t(tr("Slide ") + label);
    PageLabel->setText(t);
    importMetaData(&(PresentationBrowser3->handler->meta));
    updatePage();
}

void PoqetP::setPrevNext()
{
    if (Slider1->value() > Slider1->minValue())
    {
        prevPageAction->setIconSet(Resource::loadPixmap("back"));
    }
    else
    {
        prevPageAction->setIconSet(Resource::loadPixmap("back_inactive"));
    }
    if (Slider1->value() < Slider1->maxValue())
    {
        nextPageAction->setIconSet(Resource::loadPixmap("forward"));
    }
    else
    {
        nextPageAction->setIconSet(Resource::loadPixmap("forward_inactive"));
    }
}

void PoqetP::magnifyUpdate(int index)
{
     PresentationBrowser3->setFactor(index+1);
     switch(index)
     {
     case 0:
          PresentationBrowser3->setFullscreen(false);
          canvas.resize(PresentationBrowser3->width()-4,PresentationBrowser3->height()-4);
          PresentationBrowser3->setGeometry( QRect( 0, 0, 235, 180 ) );
          break;
     case 1:
          PresentationBrowser3->setFullscreen(false);
          canvas.resize(PresentationBrowser3->width()*2-4,PresentationBrowser3->height()*2-4);
          break;
     case 2:
          PresentationBrowser3->setFullscreen(false);
          canvas.resize(PresentationBrowser3->width()*3-4,PresentationBrowser3->height()*3-4);
          break;
     case 3:
          PresentationBrowser3->setFullscreen(true);
          canvas.resize(QPEApplication::desktop()->height(),QPEApplication::desktop()->width());
          PresentationBrowser3->setVScrollBarMode(QScrollView::AlwaysOff);
          PresentationBrowser3->setHScrollBarMode(QScrollView::AlwaysOff);
          fullscreen = true;    
     default:
          break;
     }
     PresentationBrowser3->show();
     updatePage();
}

void PoqetP::importMetaData(pXmlNode *meta)
{
    QListViewItem * element;
    if ( ! stack.isEmpty() ){
        element = new QListViewItem( stack.top(), meta->getNodeName(), meta->getText() );
    } else {
        element = new QListViewItem( MetaView, meta->getNodeName(), meta->getText() );
    }
    stack.push( element );
    element->setOpen( TRUE );
    for ( int i = 0 ; i <  meta->getA().length(); i++ )
    {
         new QListViewItem( element,  meta->getA().qName(i), meta->getA().value(i) );
    }
    QListIterator<pXmlNode> rit(meta->children);
    for ( ; rit.current(); ++rit )
    {
        pXmlNode *search = rit.current();
        importMetaData(search);
    }
    stack.pop();
}

void PoqetP::startProgressBar(int count)
{
    Progress->show(); 
    Progress->setTotalSteps(count);
    Progress->setProgress(0);
}

void PoqetP::stopProgressBar()
{
    Progress->setProgress(Progress->totalSteps());
}

void PoqetP::offFullscreenHandler()
{
     fullscreen = false;
     if (!PresentationBrowser3->parentWidget())
     {
        PresentationBrowser3->reparent(TabPB,QPoint(0,0));
        PresentationBrowser3->setVScrollBarMode(QScrollView::Auto);
        PresentationBrowser3->setHScrollBarMode(QScrollView::Auto);
        Mag->select(0);
        magnifyUpdate(0);
     }
}

void PoqetP::bitBltNow()
{    
     if (fullscreen)
     {
         PresentationBrowser3->canvas()->resize(QPEApplication::desktop()->height(),QPEApplication::desktop()->width());
         PresentationBrowser3->renderPage(Slider1->value());
         PresentationBrowser3->showFullscreenPixmap();
     }
}

void PoqetP::keyPressEvent(QKeyEvent *e)
{
     if ((e->key() == Qt::Key_Left)||(e->key() == Qt::Key_Up))
     {
         viewPrevPage();
     }
     else if ((e->key() == Qt::Key_Right)||(e->key() == Qt::Key_Down))
     {
         viewNextPage();
     }
     else
     {
         e->ignore();
     }
}

void PoqetP::timerSlot()
{
  documentOpen(currentDoc);
}
