/***************************************************************************
                          poqetp.h  -  description
                             -------------------
    begin                : Thu Jul 11 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef POQETP_H
#define POQETP_H

#include <qmainwindow.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qstack.h>
#include <qvector.h>
#include <qslider.h>
#include <qaction.h>
#include <qlabel.h>
#include <qpe/process.h>
#include "poqet.h"
#include "pbrowser.h"

/** \class PoqetP
  * \brief Descendent of PoqetPresenter. 
  * @author Ryan Rusaw
  */
class PoqetP : public PoqetPresenter
{
    Q_OBJECT

public:
    /**
      * Constructor
      * @param parent Parent widget
      * @param name widget name
      * @param fl widget flags
      */
    PoqetP( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    /**
      * Deconstructor
      */
    ~PoqetP();
    /**
      * \brief Open document
      * @param lnkDoc document to open as a DocLnk
      */
    void documentOpen(const DocLnk &lnkDoc);
    /**
      * \brief Open document
      * @param str document to open as a QString
      */
    void documentOpen(const QString &str);
    /**
      * \brief Imports document data
      * @param meta root node of meta document
      */
    void importMetaData(pXmlNode *meta);
    /**
      * \brief Copies a DocLnk
      * @param source source DocLnk
      * @param target target DocLnk
      */
    void copyDocLnk(const DocLnk &source, DocLnk &target);
    /**
      * \brief Parses xml documents into trees
      * @param lnkDoc .sxi document to parse
      */
    void parseXMLFiles(const DocLnk &lnkDoc);

public slots:

    /**
      * \brief Opens file selector
      */
    virtual void fileOpen();
    /**
      * \brief Exits
      */
    virtual void fileExit();
    /**
      * \brief Shows about dialog
      */
    virtual void helpAbout();
    /**
      * \brief Shows next page
      */
    virtual void viewNextPage();
    /**
      * \brief Shows previous page
      */
    virtual void viewPrevPage();
    /**
      * \brief Shows file selector
      */
    virtual void selectorShow();
    /**
      * \brief Hides file selector
      */
    virtual void selectorHide();
    /**
      * \brief Updates current page display
      */
    virtual void updatePage();
    /**
      * \brief Updates page
      * @param i page to update
      */
    virtual void updatePage(int i);
    /**
      * \brief Opens file
      * @param lnkDoc document to open
      */
    virtual void selectorFileOpen(const DocLnk &lnkDoc);
    /**
      * \brief Sets internal document reference name
      * @param fileref name
      */
    virtual void setDocument(const QString &fileref);
    /**
      * \brief Changes magnification and updates page
      * @param index size to change to
      */
    virtual void magnifyUpdate(int index);
    /**
      * \brief Sets up progress bar
      * @param count number of progress steps
      */
    virtual void startProgressBar(int count);
    /**
      * \brief Stops progress bar
      */
    virtual void stopProgressBar();
    /**
      * \brief Updates internal flags for non-fullscreen mode
      */
    virtual void offFullscreenHandler();
    /**
      * \brief Handles key presses passed from child widget
      * @param e event
      */
    virtual void keyPressEvent(QKeyEvent *e);
    /**
      * \brief bitBlts internal pixmap to screen
      */
    virtual void bitBltNow();

private slots:

    /**
      * \brief workaround timer to hide selector
      */
    virtual void timerSlot();

private:
    /**
      * \brief fullscreen mode flag
      */
    bool fullscreen;
    /**
      * \brief enables/disables prev/next page menu and tool buttons
      */
    void setPrevNext();
    /**
      * \brief Internal stack.
      */
    QStack<QListViewItem> stack;
    /**
      * \brief documentModified flag
      */
    bool documentModified;
};
#endif
