/***************************************************************************
                          pparser.cpp  -  description
                             -------------------
    begin                : Thu Jul 11 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pparser.h"
#include "poqet.h"
#include "pcontent.h"
#include <qstring.h>
#include <qxml.h>

PParser::PParser() : QXmlDefaultHandler ()
{
    current = NULL;
}

bool PParser::characters(const QString &ch)
{
    if ((current != NULL) && (filetype != MANIFEST))
    {
       current->setText(ch);
    }
    return TRUE;
}

bool PParser::startDocument()
{
    return TRUE;
}

bool PParser::endDocument()
{
    current = NULL;
    filetype = 0;
    return TRUE;
}

bool PParser::startElement( const QString &namespaceURI, const QString &localName,
                                        const QString& qName,
                                        const QXmlAttributes& attr)
{
    filetype = getFileType(qName);
    switch (filetype)
    {
    case MANIFEST:
        if (qName == "manifest:file-entry")
        {
            sxifile *newfile = new sxifile();
            for (int i = 0; i < attr.length(); i++)
            {
                if (attr.localName(i) == "media-type")
                {
                     newfile->mediatype = attr.value(i);
                }
                if (attr.localName(i) == "full-path")
                {
                     newfile->fullpath = attr.value(i);
                }
            }
            file.append(newfile);
        }
        return TRUE;
        break;
    case OCONTENT:
        if (current == NULL)
        {
            content.setNodeName(qName);
            content.setA(attr);
            current = &content;
            return TRUE;
        }
        break;
    case OSTYLES:
        if (current == NULL)
        {
            style.setNodeName(qName);
            style.setA(attr);
            current = &style;
            return TRUE;
        }
        break;
    case OSETTINGS:
        if (current == NULL)
        {
            settings.setNodeName(qName);
            settings.setA(attr);
            current = &settings;
            return TRUE;
        }
        break;
    case OMETA:
        if (current == NULL)
        {
            meta.setNodeName(qName);
            meta.setA(attr);
            current = &meta;
            return TRUE;
        }
        break;
    default:
        break;
    }
    if (current != NULL)
    {
          pXmlNode *newnode = new pXmlNode(current, qName, attr);
          current->children.append(newnode);
          current = newnode;
    } 
    return TRUE;
}

bool PParser::endElement( const QString& namespaceURI, const QString &localName, const QString &qName )
{
    if ((filetype != MANIFEST) && (current != NULL))
    {
          current = current->getParent();
    }
    return TRUE;
}

int PParser::getFileType(const QString &qName)
{
    if (qName == "manifest:manifest") return MANIFEST;
    if (qName == "office:document-content" ) return OCONTENT;                                                            
    if (qName == "office:document-styles" ) return OSTYLES;
    if (qName == "office:document-meta" )  return OMETA;    
    if (qName == "office:document-settings" ) return  OSETTINGS;
    return filetype;
}


