/***************************************************************************
                          pparser.h  -  description
                             -------------------
    begin                : Thu Jul 11 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef PPARSER_H
#define PPARSER_H
#include <qxml.h>
#include <qobject.h>
#include "pcontent.h"

#define MANIFEST 0
#define OCONTENT 1
#define OSTYLES 2
#define OMETA 3
#define OSETTINGS 4

class QString;

/** \class sxifile
  * \brief A class for the .sxi sub files.
  * @author Ryan Rusaw
  */
class sxifile : public QObject
{

public:
/**
  * \brief Constructor
  */
  sxifile(){};
/**
  * \brief Deconstructor
  */
  ~sxifile(){};
/**
  * \brief Type of file
  */
  QString mediatype;
/**
  * \brief path of internal file
  */
  QString fullpath;
/**
  * \brief path of internal file
  */
};

/** \class PParser
  * \brief A class to parse the xml documents and place them in the appropriate pXmlNode tree's. 
  * @author Ryan Rusaw
  */
  
class PParser : public QXmlDefaultHandler
{
public:
/**
  * \brief Constructor
  */
    PParser();
/**
  * \brief Deconstructor
  */
    ~PParser(){};
/**
  * \brief Does nothing
  * @return true
  */
    bool startDocument();
/**
  * \brief Resets current node and filetype
  * @return true
  */
    bool endDocument();
/**
  * \brief Sets text value of current node to ch
  * @return true
  */
    bool characters( const QString &ch );
/**
  * \brief Customized handler for start of xml element, inserts it into new child of current node
  * @param namespaceURI ignored
  * @param localName ignored
  * @param qName Qualified name to insert into current node
  * @param attr xml attributes
  * @return true
  */
    bool startElement( const QString& namespaceURI, const QString& localName, const QString &qName,
                       const QXmlAttributes &attr);
/**
  * \brief Customized handler for end of xml element,moves current node to parent of current.
  * @param namespaceURI ignored
  * @param localName ignored
  * @param qName ignored
  * @return true
  */
    bool endElement( const QString& namespaceURI, const QString &localName, const QString &qName );
/**
  * \brief pointer to first file
  */
    sxifile *first;
/** 
  * \brief pointer to current file
  */
    sxifile *currfile;
/**
  * \brief list of files
  */
    QList<sxifile> file;
/**
  * \brief root node of content file.
  */
    pXmlNode content;
/**
  * \brief root node of style file
  */
    pXmlNode style;
/**
  * \brief root node of settings file
  */
    pXmlNode settings;
/**
  * \brief root node of meta file
  */
    pXmlNode meta;
/**
  * \brief pointer to current node
  */
    pXmlNode *current;
/**
  * \brief list of images
  */
    QList<pImage> images; 

private:
/**
  * \brief Gets the file type of file qith qName
  * @param qName qualifed name of the file from its xml structure.
  * @return filetype of qName or current filename if not found
  */
    int getFileType( const QString &qName);
/**
  * \brief current filetype of current file
  */
    int filetype;

};

#endif
