/***************************************************************************
                          putils.h  -  description
                             -------------------
    begin                : Fri Aug 23 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef PUTILS_H
#define PUTILS_H

#include <qstring.h>
#include <qobject.h>
#include <qimage.h>
#include <qasciidict.h>

/** \class pUtils
  * \brief Utility function class.
  * @author Ryan Rusaw
  */
 
class pUtils : public QObject
{
public:
     /**
       * \brief Constructor
       */
     pUtils();
     /**
       * \brief Deconstructor
       */
     ~pUtils(){};
     /**
       * \brief Generates roman numerals for a number.
       * @param num The number
       * @param uppercase Generates uppercase numeral if true
       * @return The numeral as a QString
       */
     static QString roman(int num,bool uppercase);
     /**
       * \brief Gets the Image name based off the hashed URL value.
       * @param hrefname Key to search for in dictionary
       * @return The value for the given key
       */  
     static QString getImageName(QString *hrefname);
     /**
       * \brief Generates a square root to the nearest integer
       * @param val The value to find the qsuare root of.
       * @return the qsuare root to the nearest integer
       */
     static unsigned int intsqrt(unsigned long val );
     /**
       * \brief Returns sin of an angle
       * @param val angle as a 10 * angle in degrees to the nearest integer.
       * @return the sin*1000 to the nearest integer
       */
     static int intsin(int val);
     /**
       * \brief Returns cos of an angle
       * @param val angle as a 10 * angle in degrees to the nearest integer.
       * @return the cos*1000 to the nearest integer
       */
     static int intcos(int val);
     /**
       * \brief Returns tan of an angle
       * @param val angle as a 10 * angle in degrees to the nearest integer.
       * @return the tan*1000 to the nearest integer
       */
     static int inttan(int val);
     /**
       * \brief Returns square root of a number between 0 and 1
       * @param val number * 1000 to the nearest integer.
       * @return square root * 1000 to the nearest integer
       */
     static int qfsqrt(int val);
private:
     /**
       * \brief Generates roman numerals. Internal use only.
       */
     static int genroman(int i, int j, char c, QString *s);
     /**
       * \brief Ascii dictionary of image names by hrefname.
       */                                                                    
     static QAsciiDict<char> *dict;                                        
};
#endif

