/***************************************************************************
                          renderbg.cpp  -  description
                             -------------------
    begin                : Mon Sep 2 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "renderbg.h"

RenderBG::RenderBG(QObject *parent, QCanvas *canvas, pXmlNode *contentroot, pXmlNode *styleroot, QList<pImage> *i)
     : RenderItems(parent, canvas, contentroot, styleroot, i)
{
}

void RenderBG::render(pXmlNode *node)
{
    QString stylename = findStyleProperty(node,"draw:style-name");
    QString fill = findStyleProperty(node,"draw:fill");
    if ( fill != NULL)
    {
         if (fill == "bitmap")
         {
               QString bmpname = findStyleProperty(node,"draw:fill-image-name");
               pXmlNode *temp = style->findChildWithNameAndValue("draw:fill-image","draw:name", bmpname);
               QString fname = temp->getA().value("xlink:href");
               pImage *data = images.first();
               while (((QString)"#" + data->filename) != fname)
               {
                    data = images.next();
                    if (data == NULL)
                    {
                         continue;
                    }
               }
               QImage *i = data->image;
               if (!i)
               {
                    i = new QImage(Resource::loadImage(pUtils::getImageName(&fname)));
               }
               i->setAlphaBuffer(TRUE);
               QPixmap *pm = new QPixmap();
               int w = (int)(i->width()/SCALE_FACTOR/factor);
               int h = (int)(i->height()/SCALE_FACTOR/factor);
               pm->convertFromImage(i->smoothScale(w,h),0);
               canvas()->setBackgroundPixmap(*pm);
         }
         else if (fill == "solid")
         {
              QColor *color = new QColor( findStyleProperty(node,"draw:fill-color"));
              canvas()->setBackgroundColor(*color);
         }
         else if (fill == "gradient")
         {
              QString gradname = findStyleProperty(node,"draw:fill-gradient-name");
              pXmlNode *gradientnode = style->findChildWithNameAndValue("draw:gradient","draw:name",gradname);
              int step = findStyleProperty(node,"draw:gradient-step-count").toInt();
              if (step == 0 ) step = 64;
              QPixmap *gradpm = new QPixmap();
              QImage *i = new QImage(*makeGradientImage(gradientnode,step));
              i->setAlphaBuffer(TRUE);
              gradpm->convertFromImage(*i);
              if (gradpm != NULL)
                   canvas()->setBackgroundPixmap(*gradpm);
         }
         else if (fill == "hatch")
         {
              QColor *color = new QColor( findStyleProperty(node, "draw:fill-color"));
              canvas()->setBackgroundColor(*color);
              QString hatchname = findStyleProperty(node,"draw:fill-hatch-name");
              pXmlNode *hatchnode = style->findChildWithNameAndValue("draw:hatch","draw:name",hatchname);
              QPixmap *hatchpm = new QPixmap();
              QImage *i = new QImage(*makeHatchImage(hatchnode,color));
              i->setAlphaBuffer(TRUE);
              hatchpm->convertFromImage(*i);
              if (hatchpm != NULL)
                   canvas()->setBackgroundPixmap(*hatchpm);
         }
    }
}