/***************************************************************************
                          renderellipse.cpp  -  description
                             -------------------
    begin                : Mon Sep 2 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "renderellipse.h"
#include "rendertxt.h"

RenderEllipse::RenderEllipse(QObject *parent, QCanvas *canvas, pXmlNode *contentroot, pXmlNode *styleroot, QList<pImage> *i)
     : RenderItems(parent, canvas, contentroot, styleroot, i)
{
}

void RenderEllipse::render(pXmlNode* node)
{
    int x,y,w,h;
    QCanvasLine *outline;
    x = (int)((findStyleProperty(node,"svg:x").remove(findStyleProperty(node,"svg:x").length()-2,2).toFloat())*CM_TO_PXL/factor);
    y = (int)((findStyleProperty(node,"svg:y").remove(findStyleProperty(node,"svg:y").length()-2,2).toFloat())*CM_TO_PXL/factor);
    w = (int)((findStyleProperty(node,"svg:width").remove(findStyleProperty(node,"svg:width").length()-2,2).toFloat())*CM_TO_PXL/factor);
    h = (int)((findStyleProperty(node,"svg:height").remove(findStyleProperty(node,"svg:height").length()-2,2).toFloat())*CM_TO_PXL/factor);
    int startangle = (int)(findStyleProperty(node, "draw:start-angle").toFloat()*16);
    int endangle = (int)(findStyleProperty(node, "draw:end-angle").toFloat()*16);
    int arc;
    if ( endangle > startangle)
       arc = endangle-startangle;
    else
       arc = 5760+endangle-startangle;
    QPointArray linepoints;
    if ( findStyleProperty(node, "draw:kind") == "cut" )
    {
       CanvasCutEllipse* cutellipse;
       cutellipse = new CanvasCutEllipse(w,h,startangle,arc,canvas());
       linepoints.makeArc(x,y,w,h,startangle,arc);

       if ( findStyleProperty(node, "draw:fill") == "solid")
       {
            cutellipse->setBrush(QColor(findStyleProperty(node,"draw:fill-color")));
       }
       else
       {
            if (replaceitems)
            {
               cutellipse->setBrush(QColor("#DCDCDC"));
            }
       }
       cutellipse->move(x+(w/2),y+(h/2));
       cutellipse->setZ(layer++);
       cutellipse->show();
    }
    else if ( findStyleProperty(node, "draw:kind" ) == "arc" )
    {
            // arcs have no fill;
            linepoints.makeArc(x,y,w,h,startangle,arc);
    }
    else if (findStyleProperty(node, "draw:kind") == "section" )
    {
       QCanvasEllipse* sectellipse;
       sectellipse = new QCanvasEllipse(w,h,startangle,arc,canvas());
       linepoints.makeArc(x,y,w,h,startangle,arc);
       linepoints.setPoint(linepoints.size()-1,x+(w/2),y+(h/2));
       if ( findStyleProperty(node, "draw:fill") == "solid")
       {
            sectellipse->setBrush(QColor(findStyleProperty(node,"draw:fill-color")));
       }
       else
       {
            if (replaceitems)
            {
               sectellipse->setBrush(QColor("#DCDCDC"));
            }
       }
       sectellipse->move(x+(w/2),y+(h/2));
       sectellipse->setZ(layer++);
       sectellipse->show();
    }
    else
    {
       QCanvasPolygonalItem* ellipse;
       ellipse = new QCanvasEllipse(w,h,canvas());
       if ( findStyleProperty(node, "draw:fill") == "solid")
       {
            ellipse->setBrush( QColor(findStyleProperty(node,"draw:fill-color")));
       }
       else
       {
            if (replaceitems)
            {
               ellipse->setBrush(QColor("#DCDCDC"));
            }
       }
       ellipse->move(x+(w/2),y+(h/2));
       linepoints.makeEllipse(x,y,w,h);
       ellipse->setZ(layer++);
       ellipse->show();
     }
    /* stupid no pen :( */
    for (uint i = 0; i < linepoints.size(); i++ )
    {
            QString c = findStyleProperty(node,"svg:stroke-color");
            outline = new QCanvasLine(canvas());
            int myx, myy, mx2, my2;
            if (i < linepoints.size()-1)
            {
                 myx = linepoints.point(i).x();
                 myy = linepoints.point(i).y();
                 mx2 = (linepoints.point(i+1).x());
                 my2 = (linepoints.point(i+1).y());
                 outline->setPoints(myx,myy,mx2,my2);
            }
            else
            {
                 if ( findStyleProperty(node, "draw:kind" ) != "arc")
                 {
                      myx = linepoints.point(i).x();
                      myy = linepoints.point(i).y();
                      mx2 = (linepoints.point(0).x());
                      my2 = (linepoints.point(0).y());
                      outline->setPoints(myx,myy,mx2,my2);
                 }
            }
            if ( c.length() > 0 )
            {
                 outline->setPen(QColor(c));
            }
            else
            {
                 outline->setPen(QPen());
            }
            outline->setZ(layer);
            outline->show();
    }
    layer++;
    ((RenderTxt*)this)->render(node);
}