/***************************************************************************
                          renderitems.cpp  -  description
                             -------------------
    begin                : Thu Aug 1 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "renderitems.h"
#include <qvector.h>

RenderItems::RenderItems(QObject *parent, QCanvas *canvas, pXmlNode *contentroot, pXmlNode *styleroot, QList<pImage> *i) : QObject(parent)
{
     thiscanvas = canvas;
     content = contentroot;
     style = styleroot;
     images = *i;
     lbcount = 0;
     listinc = false;
     lastbottom = 0;
     textlvl = 0;
     rect = NULL;
}

void RenderItems::render(pXmlNode* node)
{
}

void RenderItems::xformRadialGradiant(QImage *image, int steps, int radius, int cx, int cy, QRgb start, QRgb end, bool blendAlpha )
{
    int sr,sg,sb,sa,er,eg,eb,ea,dr,dg,db,da;
    int n;
    int w = image->width();
    int h = image->height();
    sr = qRed(start); sg = qGreen(start); sb = qBlue(start); sa = qAlpha(start);
    er = qRed(end); eg = qGreen(end); eb = qBlue(end); ea = qAlpha(end);
    dr = (er-sr)*100/(steps-1);  dg = (eg-sg)*100/(steps-1);
    db = (eb-sb)*100/(steps-1);  da = (ea-sa)*100/(steps-1);
    QArray<QRgb> colors = QArray<QRgb>(steps);
    for ( int count = 0; count < steps; count++)
    {
        int r = sr+int(dr*(steps-count-1)/100);
        int g = sg+int(dg*(steps-count-1)/100);
        int b = sb+int(db*(steps-count-1)/100);
        int a = sa+int(da*(steps-count-1)/100);
        colors[count] = qRgba(r,g,b,a);
    }
    int k = radius*1000/(steps-1);
    int z = (radius*1000-k)/(steps-1);
    for (int y=0; y < h; y++)
    {
        QRgb* scanln = (QRgb*)image->scanLine(y);
        int tempy = (y-cy)*(y-cy);
        for (int x = 0; x < w; x++)
        {
            n = pUtils::intsqrt((x-cx)*(x-cx)+tempy)*1000/z;
            if (n > (steps-1)) n = (steps-1);
            if (!blendAlpha)
               *(scanln+x) = colors.data()[n];
            else
               *(scanln+x) = image->pixel(x,y) & colors.data()[n];
        }
    }
}

void RenderItems::xformAxialGradiant(QImage *image, int steps, int y2, int mm, int axiald, QRgb start, QRgb end, bool blendAlpha)
{
    int sr,sg,sb,sa,er,eg,eb,ea,dr,dg,db,da;
    int U, n = 0, px, py, distance;
    sr = qRed(start); sg = qGreen(start); sb = qBlue(start); sa = qAlpha(start);
    er = qRed(end); eg = qGreen(end); eb = qBlue(end); ea = qAlpha(end);
    dr = (er-sr)*100/(steps-1);  dg = (eg-sg)*100/(steps-1);
    db = (eb-sb)*100/(steps-1);  da = (ea-sa)*100/(steps-1);
    QArray<QRgb> colors = QArray<QRgb>(steps);
    for ( int count = 0; count < steps; count++)
    {
        int r = sr+int(dr*(steps-count-1)/100);
        int g = sg+int(dg*(steps-count-1)/100);
        int b = sb+int(db*(steps-count-1)/100);
        int a = sa+int(da*(steps-count-1)/100);
        colors[count] = qRgba(r,g,b,a);
    }
    int w = image->width();
    int h = image->height();
    for (int y=0; y < h; y++)
    {
         QRgb* scanln = (QRgb*)image->scanLine(y);
         int tempy = (h+y2-y) * (2*y2+h);
         for (int x = 0; x < w; x++)
         {
              U = int(((x*w)+tempy));
              if ( (U > 0) && (U < mm))
              {
                   px = U*w/mm;
                   py = (U*(h+2*y2))/mm - (h+y2);
                   distance = pUtils::intsqrt((px-x)*(px-x)+(py+y)*(py+y));
                   n = (int)((distance*1024/axiald)*(steps-1)>>10);
                   if ( n > (steps-1) ) n = (steps-1);
              }
              if (!blendAlpha)
                  *(scanln+x) = colors.data()[steps-n-1];
              else
                  *(scanln+x) = image->pixel(x,y) & colors.data()[steps-n-1];;
         }                                                       
    }
}

void RenderItems::xformLinearGradiant(QImage *image, int steps, int angle, QRgb start, QRgb end, bool blendAlpha )
{
     int sr,sg,sb,sa,er,eg,eb,ea,dr,dg,db,da;
     int U, n=0, px, py, distance;
     sr = qRed(start); sg = qGreen(start); sb = qBlue(start); sa = qAlpha(start);
     er = qRed(end); eg = qGreen(end); eb = qBlue(end); ea = qAlpha(end);
     dr = (er-sr)*100/(steps-1);  dg = (eg-sg)*100/(steps-1);
     db = (eb-sb)*100/(steps-1);  da = (ea-sa)*100/(steps-1);
     QArray<QRgb> colors = QArray<QRgb>(steps);
     for ( int count = 0; count < steps; count++)
     {
         int r = sr+int(dr*(steps-count-1)/100);
         int g = sg+int(dg*(steps-count-1)/100);
         int b = sb+int(db*(steps-count-1)/100);
         int a = sa+int(da*(steps-count-1)/100);
         colors[count] = qRgba(r,g,b,a);
     }
     int w = image->width();
     int h = image->height();
     if ((angle % 900) != 0)
     {
          int x2 = 1000*h/pUtils::inttan(angle);
          int y2 = w*pUtils::inttan(angle)/1000;
          int x3 = w + x2;
          int y3 = h + y2;
          int xy2 = (x3*x3+y3*y3);
          int zl = pUtils::intsqrt((h*h)+(w*w));
          for (int y=0; y < h; y++)
          {
               QRgb* scanln = (QRgb*)image->scanLine(y);
               int tempy = (h-y)*y3;
               for (int x = 0; x < w; x++)
               {
                 U = (x+x2)*x3+tempy;
                 if ( (U > 0) && (U < xy2))
                 {
                    px = int(U*(x2+w)/xy2)-x2;
                    py = int(U*(y2+h)/xy2)-h;
                    distance = pUtils::intsqrt(((px-x)*(px-x))+((py+y)*(py+y)));
                    n = int((distance*1024/zl)*(steps-1)>>10);
                    if (n > (steps-1)) n = (steps-1);
                 }
                 if (!blendAlpha)
                     *(scanln+x) = colors.data()[steps-n-1];
                 else
                     *(scanln+x) = image->pixel(x,y) & colors.data()[steps-n-1];
               }
          }
     }
     else
     {
          for (int y=0; y < h; y++)
          {
               QRgb* scanln = (QRgb*)image->scanLine(y);
               for (int x = 0; x < w; x++)
               {
                   if (angle == 900)
                   {
                       n = int(((w-x)*1024/w)*(steps-1)>>10);
                   }
                   else if (angle == 1800)
                   {
                       n = int(((h-y)*1024/h)*(steps-1)>>10);
                   }
                   else if (angle == 2700)
                   {
                       n = int((x*1024/w)*(steps-1)>>10);
                   }
                   else
                   {
                       n = int((y*1024/h)*(steps-1)>>10);
                   }
                   if (!blendAlpha)
                       *(scanln+x) = colors.data()[steps-n-1];
                   else
                       *(scanln+x) = image->pixel(x,y) & colors.data()[steps-n-1];
               }
          }
     }
}

void RenderItems::xformEllipsoidGradiant(QImage *image, int steps, int cx, int cy, int angle, int radius, QRgb start, QRgb end, bool blendAlpha )
{
    int sr,sg,sb,sa,er,eg,eb,ea,dr,dg,db,da;
    int distance, distance2, n;
    int w = image->width();
    int h = image->height();
    sr = qRed(start); sg = qGreen(start); sb = qBlue(start); sa = qAlpha(start);
    er = qRed(end); eg = qGreen(end); eb = qBlue(end); ea = qAlpha(end);
    dr = (er-sr)*100/(steps-1);  dg = (eg-sg)*100/(steps-1);
    db = (eb-sb)*100/(steps-1);  da = (ea-sa)*100/(steps-1);
    QArray<QRgb> colors = QArray<QRgb>(steps);
    for ( int count = 0; count < steps; count++)
    {
        int r = sr+int(dr*(steps-count-1)/100);
        int g = sg+int(dg*(steps-count-1)/100);
        int b = sb+int(db*(steps-count-1)/100);
        int a = sa+int(da*(steps-count-1)/100);
        colors[count] = qRgba(r,g,b,a);
    }
    int cx2 = (radius*pUtils::intcos(angle)/4000)+cx;
    int cy2 = (radius*pUtils::intsin(angle)/4000)+cy;
    int cx3 = cx-(radius*pUtils::intcos(angle)/4000);
    int cy3 = cy-(radius*pUtils::intsin(angle)/4000);
    int axiald = (pUtils::intsqrt(cx2*cx2+cy2*cy2) + pUtils::intsqrt(cx3*cx3+cy3*cy3));
    for (int y=0; y < h; y++)
    {
         int tempy = (cy2-y)*(cy2-y);
         int tempy2 = (cy3-y)*(cy3-y);
         QRgb* scanln = (QRgb*)image->scanLine(y);
         for (int x = 0; x < w; x++)
         {
           distance = pUtils::intsqrt((cx2-x)*(cx2-x)+tempy);
           distance2 = pUtils::intsqrt((cx3-x)*(cx3-x)+tempy2);
           n = ((distance+distance2)*(steps-1))/axiald;
           if (n > (steps-1)) n = (steps-1);
           if (!blendAlpha)
              *(scanln+x) = colors.data()[n];
           else
              *(scanln+x) = image->pixel(x,y) & colors.data()[n];
         }
    }
}

void RenderItems::xformAlphaChannel(QImage* image, QString *alphastyle)
{
    QString temp;
    pXmlNode *curr;
    int alphas, alphae, alphamode, angle, cx, cy;
    if ( alphastyle == NULL) return;
    if ( alphastyle->length() == 0 ) return;
    if ( ! alphastyle->contains("%") )
    {
         curr = style->findChildWithNameAndValue("draw:transparency","draw:name", *alphastyle, TRUE);
         temp = curr->getA().value("draw:start");
         temp.truncate(temp.length()-1);
         alphas = 255 - (temp.toInt()*255)/100;
         temp = curr->getA().value("draw:end");
         temp.truncate(temp.length()-1);
         alphae = 255 - (temp.toInt()*255)/100;
         temp = curr->getA().value("draw:style");
         if ( temp == "linear" ) alphamode = LinGradAlphaMode;
         else if ( temp == "radial" ) alphamode = RadGradAlphaMode;
         else if ( temp == "axial" ) alphamode = AxGradAlphaMode;
         else if ( temp == "ellipsoid" ) alphamode = EllGradAlphaMode;
         else alphamode = NoAlphaMode;
    }
    else // just flat style
    {
         temp = *alphastyle;
         temp.truncate(temp.length()-1);
         alphas = 255 - (temp.toInt()*255)/100;
         alphae = alphas;
         alphamode = FlatAlphaMode;
    }
    QRgb start = qRgba( 0xFF, 0xFF, 0xFF, alphas );
    QRgb stop = qRgba( 0xFF, 0xFF, 0xFF, alphae );
    int w, h, y2, x2, axiald, m;
    switch (alphamode)
    {
    case FlatAlphaMode:
         image->setAlphaBuffer(TRUE);
         for (int yy=0; yy<image->height(); yy++)
         {
              for (int xx = 0; xx <image->width(); xx++)
              {
                 QRgb col = image->pixel(xx,yy);
                 QRgb alphacol = qRgba( qRed( col ), qGreen( col ), qBlue( col ), alphas );
                 image->setPixel( xx, yy, alphacol );
              }
         }
         break;
    case LinGradAlphaMode:
         temp = curr->getA().value("draw:angle");
         angle = temp.toInt();
         image->setAlphaBuffer(TRUE);
         xformLinearGradiant(image, 48, angle, start, stop, true);
         break;
    case RadGradAlphaMode:
         image->setAlphaBuffer(TRUE);
         temp = curr->getA().value("draw:cx");
         temp.truncate(temp.length()-1);;
         cx = temp.toInt()*image->width()/100;
         temp = curr->getA().value("draw:cy");
         temp.truncate(temp.length()-1);;
         cy = temp.toInt()*image->height()/100;
         if (cx > cy) xformRadialGradiant(image, 48, cx, cx, cy, start, stop, true);
         else xformRadialGradiant(image, 48, cy, cx, cy, start, stop, true );
         break;
    case AxGradAlphaMode:
         temp = curr->getA().value("draw:angle");
         angle = temp.toInt();
         image->setAlphaBuffer(TRUE);
         w = image->width();
         h = image->height();
         y2 = pUtils::inttan(angle)*(w/2)/1000-h/2;
         m = pUtils::intsqrt((2*y2+h)*(2*y2+h)+w*w);
         if ((angle % 1800) == 0)
         {
               axiald = h/2;
         }
         else if ((angle % 900) == 0)
         {
               axiald = w/2;
         }
         else
         {
               x2 = (int)pUtils::intsqrt(w*h/pUtils::inttan(angle));
               axiald = (int)((w-x2)*pUtils::inttan(angle)/1500);
         }
         xformAxialGradiant(image,48,y2,m*m,axiald,start,stop,true);
         break;
    case EllGradAlphaMode:
         image->setAlphaBuffer(TRUE);
         temp = curr->getA().value("draw:cx");
         temp.truncate(temp.length()-1);;
         cx = temp.toInt()*image->width()/100;
         temp = curr->getA().value("draw:cy");
         temp.truncate(temp.length()-1);;
         cy = temp.toInt()*image->height()/100;
         temp = curr->getA().value("draw:angle");
         angle = temp.toInt();
         if (cx > cy) xformEllipsoidGradiant(image, 48, cx, cy, angle, 10, start, stop, true);
         else xformEllipsoidGradiant(image, 48, cx, cy, angle, 10, start, stop, true );
         break;
    default:
         // TODO
         break;     
    }
}

QImage *RenderItems::makeGradientImage(pXmlNode *gradnode, int steps, QSize *size)
{
     int m=0;
     float pi = 3.14159265359;
     int x2=0,y2=0,cx,cy,radius,w,h;
     int axiald,angle;
     QImage *temp ;
     
     if (size == NULL)
          temp = new QImage(canvas()->width(),canvas()->height(),32,255);
     else
          temp = new QImage(size->width(),size->height(),32,255);
     w = temp->width();
     h = temp->height();
     temp->setAlphaBuffer(TRUE);
     
     QColor *start = new QColor(findStyleProperty(gradnode,"draw:start-color"));
     QColor *stop = new QColor(findStyleProperty(gradnode,"draw:end-color"));
     
     QString type = findStyleProperty(gradnode,"draw:style");
     angle = findStyleProperty(gradnode,"draw:angle").toInt();
     if ( type == "radial")
     {
          cx = (int)(findStyleProperty(gradnode,"draw:cx").toFloat()/100*canvas()->width());
          cy = (int)(findStyleProperty(gradnode,"draw:cy").toFloat()/100*canvas()->height());
          radius = pUtils::intsqrt(1/(100*pi)*w*h*(100-findStyleProperty(gradnode,"draw:border").toInt()));
          xformRadialGradiant(temp,steps,radius,cx,cy,start->rgb(),stop->rgb());
     }
     else if (type == "linear")
     {
          xformLinearGradiant(temp,steps,angle,start->rgb(),stop->rgb());
     }
     else if ( type == "axial" )
     {
          y2 = (pUtils::inttan(angle)*(w/2)/1000)-h/2;
          m = pUtils::intsqrt((2*y2+h)*(2*y2+h)+w*w);
          if ( findStyleProperty(gradnode,"draw:angle") == "0")
               axiald = (int)(findStyleProperty(gradnode,"draw:border").toFloat()*100*h/20000);
          else if ( findStyleProperty(gradnode,"draw:angle") == "900")
               axiald = (int)(findStyleProperty(gradnode,"draw:border").toFloat()*100*w/20000);
          else
          {
               x2 = (int)pUtils::intsqrt(10*w*h*findStyleProperty(gradnode,"draw:border").toFloat()/pUtils::inttan(angle));
               axiald = (int)((w-x2)*pUtils::inttan(angle)/1000);
          }
          xformAxialGradiant(temp,steps,y2,m*m,axiald,start->rgb(),stop->rgb());
     }
     else if ( type == "ellipsoid" )
     {
          cx = (int)(findStyleProperty(gradnode,"draw:cx").toFloat()/100*canvas()->width());
          cy = (int)(findStyleProperty(gradnode,"draw:cy").toFloat()/100*canvas()->height());
          radius = (int)pUtils::intsqrt(w/2*h/2*(100-findStyleProperty(gradnode,"draw:border").toFloat())/100);
          xformEllipsoidGradiant(temp, steps, cx, cy, angle, radius, start->rgb(), stop->rgb());
     }
     else if (( type == "rectangular") || (type == "square")) // unimplemented.. just a vertical gradient for now
     {
          for (int y=0; y<temp->height(); y++)
          {
               for (int x = 0; x < temp->width(); x++)
               {
                   *((QRgb*)temp->scanLine(y)+x) = qRgb(255,255,255);
               }
          }
          if (replaceitems)
          {
               QPixmap *pm = new QPixmap(Resource::loadPixmap("poqetp/grey_square"));
               QImage i = pm->convertToImage();
               return &i;
          }
     }
     return temp;
}

QImage *RenderItems::makeHatchImage(pXmlNode *hatchnode, QColor *color, int tpcy, QSize *size)
{
     QImage *temp; 
     if (size == NULL)
          temp = new QImage(canvas()->width(),canvas()->height(),32,255);
     else
          temp = new QImage(size->width(),size->height(),32,255);
     temp->setAlphaBuffer(TRUE);
     float dist = findStyleProperty(hatchnode,("draw:distance")).toFloat();
     int n=0;
     bool backgrnd;
     QColor *fgcolor = new QColor(findStyleProperty(hatchnode,"draw:color"));
     QRgb fgc = fgcolor->rgb();
     QRgb orig = color->rgb();
     QRgb alpha = qRgba( qRed(orig), qGreen(orig), qBlue(orig), tpcy );
     QRgb alpha2 = qRgba( qRed(fgc), qGreen(fgc), qBlue(fgc), tpcy );
     QString type = findStyleProperty(hatchnode,"draw:name");
     QString dstyle = findStyleProperty(hatchnode,"draw:style");
     int angle = (int)(findStyleProperty(hatchnode,"draw:rotation").toInt()/10);
     int dpp = (int)(dist*CM_TO_PXL/factor);
     if ( angle%90 == 0 )
     {
          if (dpp < 4) dpp = 4;
     }
     else
     {
          if (dpp < 8) dpp = 8;
     }
     for (int y=0; y<temp->height(); y++)
     {
          for (int x = 0; x < temp->width(); x++)
          {
              if (dstyle == "single")
              {
                   if ( (angle == 45 ) || (angle == 225 ))
                   {
                       if ((x+n)%dpp == 0)
                       {
                           backgrnd = false;
                       }
                       else
                       {
                           backgrnd = true;
                       }
                   }
                   else if ( ( angle == 0 ) || (angle == 180 ))
                   {
                       if ((y)%dpp == 0)
                       {
                           backgrnd = false;
                       }
                       else
                       {
                           backgrnd = true;
                       }
                   }
                   else if ( ( angle == 135 ) || (angle == 315))
                   {
                       if ((x-n)%dpp == 0)
                       {
                           backgrnd = false;
                       }
                       else
                       {
                           backgrnd = true;
                       }
                   }
                   else if ( (angle == 90) || (angle == 270))
                   {
                       if ((x)%dpp == 0)
                       {
                           backgrnd = false;
                       }
                       else
                       {
                           backgrnd = true;
                       }
                   }
              }
              else if ( dstyle == "double" )
              {
                   if ( (angle == 45 ) || (angle == 225 ) || (angle == 135) || (angle == 315))
                   {
                       if (((x+n)%dpp == 0)||((x-n)%dpp == 0))
                       {
                           backgrnd = false;
                       }
                       else
                       {
                           backgrnd = true;
                       }
                   }
                   else
                   {
                       if (((x)%dpp == 0)||((y)%dpp == 0))
                       {
                           backgrnd = false;
                       }
                       else
                       {
                           backgrnd = true;
                       }
                   }
               }
               else if ( dstyle == "triple" )
               {
                   if ( (angle == 45 ) || (angle == 225 ))
                   {
                       if (((x)%dpp == 0) || ((x-n)%dpp == 0) || ((x+n)%dpp == 0))
                       {
                           backgrnd = false;
                       }
                       else
                       {
                           backgrnd = true;
                       }
                   }
                   else if ( ( angle == 0 ) || (angle == 180 ))
                   {
                       if (((y)%dpp == 0) || ((x)%dpp == 0) || ((x+n)%dpp == 0))
                       {
                           backgrnd = false;
                       }
                       else
                       {
                           backgrnd = true;
                       }
                   }
                   else if ( ( angle == 135 ) || (angle == 315))
                   {
                       if (((y)%dpp == 0) || ((x-n)%dpp == 0) || ((x+n)%dpp == 0))
                       {
                           backgrnd = false;
                       }
                       else
                       {
                           backgrnd = true;
                       }
                   }
                   else if ( (angle == 90) || (angle == 270))
                   {
                       if (((y)%dpp == 0) || ((x)%dpp == 0) || ((x-n)%dpp == 0))
                       {
                           backgrnd = false;
                       }
                       else
                       {
                           backgrnd = true;
                       }
                   }
               }
               if (backgrnd)
               {
                   temp->setPixel(x,y,alpha);
               }
               else
               {
                   if (tpcy == 0x00)
                   {
                       temp->setPixel(x,y,fgc);
                   }
                   else
                   {
                       temp->setPixel(x,y,alpha2);
                   }
               }
          }
          if (++n == dpp) n = 0;
     }
     return temp;
}

void RenderItems::showImage( int x, int y, int w, int h, QImage *image, QString *alphastyle)
{
     QList<QPixmap> pml;
     QCanvasPixmap *pm;
     QImage scaledimage;
     scaledimage = image->smoothScale(w,h);
     if (alphastyle != NULL)
     {
         xformAlphaChannel(&scaledimage, alphastyle);
     }
     pm = new QCanvasPixmap(scaledimage);
     pml.append(pm);
     QList<QPoint> ptl;
     ptl.append(new QPoint(0,0));
     QCanvasPixmapArray *pma = new QCanvasPixmapArray(pml, ptl);
     QCanvasSprite *sprite = new QCanvasSprite(pma,canvas());
     sprite->move(x,y);
     sprite->setZ(layer++);
     sprite->show();
}

QString RenderItems::findStyleProperty(pXmlNode *node, const QString &val)
{
     QString rstring = node->getA().value(val);
     pXmlNode *current, *tempstyle;
     if (rstring.length() > 0)
     {
          return rstring;
     }
     // search styles
     if ( node->getA().value("draw:style-name").length() > 0 )                   // graphic objects
          current = content->findChildWithNameAndValue("style:style","style:name",node->getA().value("draw:style-name"), TRUE);
     else if ( node->getA().value("presentation:style-name").length() > 0)       // text boxes
          current = content->findChildWithNameAndValue("style:style","style:name",node->getA().value("presentation:style-name"), TRUE);
     else
     {                                                                       // actual text has a style too
          QString name(node->getA().value("text:style-name"));
          current = content->findChildWithNameAndValue("style:style","style:name",name, TRUE);
          tempstyle = current;
     }
     if (current != NULL)
     {
          if (!current->children.isEmpty())
          {
               rstring = current->children.first()->getA().value(val);
               if (rstring.length() > 0)
               {
                      return rstring;
               }
          }
          rstring = current->getA().value(val);
          if (rstring.length() > 0)
          {
               return rstring;
          }
     }
     else
     {
          return "";
     }
     // search all parent styles now
     while ((current != NULL) && (current->getA().value("style:parent-style-name").length() > 0 ))
     {
           current = style->findChildWithNameAndValue("style:style","style:name",current->getA().value("style:parent-style-name"));
           if (current != NULL)
           {
               if (!current->children.isEmpty())
               {
                    rstring = current->children.first()->getA().value(val);
                    if (rstring.length() > 0)
                    {
                           return rstring;
                    }
               }
               rstring = current->getA().value(val);
               if (rstring.length() > 0)
               {
                      return rstring;
               }
           }
           else
           {
               continue;
           }
     }
     // finally search the page master style
     current = style->findChildWithNameAndValue("style:master-page","style:master-page-name",node->getA().value("draw:master-style-name"));
     if ( current != NULL )
          current = style->findChildWithNameAndValue("style:style","style:name",current->getA().value("draw:style-name"));
     else
          current = style->findChildWithNameAndValue("style:style","style:name",tempstyle->getA().value("draw:style-name"));
     if (current != NULL)
     {
         if (!current->children.isEmpty())
         {
              rstring = current->children.first()->getA().value(val);
              if (rstring.length() > 0)
              {
                     return rstring;
              }
         }
         rstring = current->getA().value(val);
         if (rstring.length() > 0)
         {
                return rstring;
         }
     }
     // did not find it.
     return "";

}

/* Deprecated alias for findStyleProperty() */

QString RenderItems::findStylePropertyInStyleRecursive(pXmlNode *node, const QString &val)
{
     QString k = findStyleProperty(node, val);
     return k;
}

QString RenderItems::findStylePropertyWithParentDelimiter(pXmlNode *node, const QString &delimiter, const QString &val)
{
     QString tmpstring = findStyleProperty(node,val);
     if (tmpstring.length() > 0)
         return tmpstring;
     else
     {
         if (node->getNodeName() != "document:content")
         {
             pXmlNode *p = node->getParent();
             while ((p != NULL) && ( p->getNodeName() != delimiter ))
             {
                  tmpstring = findStyleProperty(p,val);
                  if ( tmpstring.length() > 0)
                  {
                       return tmpstring;
                  }
                  else
                  if (p->getNodeName() != "document:content")
                  {
                      p = p->getParent();
                  }
                  else
                  {
                      continue;
                  }
             }
         }
     }
     return "";
}

