/***************************************************************************
                          renderrect.cpp  -  description
                             -------------------
    begin                : Mon Sep 2 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "renderrect.h"
#include "rendertxt.h"

RenderRect::RenderRect(QObject *parent, QCanvas *canvas, pXmlNode *contentroot, pXmlNode *styleroot, QList<pImage> *i)
     : RenderItems(parent, canvas, contentroot, styleroot, i)
{
}

void RenderRect::render(pXmlNode* node)
{
     int x,y,w,h;
     
     x = (int)((findStyleProperty(node,"svg:x").remove(findStyleProperty(node,"svg:x").length()-2,2).toFloat())*CM_TO_PXL/factor);
     y = (int)((findStyleProperty(node,"svg:y").remove(findStyleProperty(node,"svg:y").length()-2,2).toFloat())*CM_TO_PXL/factor);
     w = (int)((findStyleProperty(node,"svg:width").remove(findStyleProperty(node,"svg:width").length()-2,2).toFloat())*CM_TO_PXL/factor);
     h = (int)((findStyleProperty(node,"svg:height").remove(findStyleProperty(node,"svg:height").length()-2,2).toFloat())*CM_TO_PXL/factor);
     if (findStyleProperty(node,"draw:corner-radius").toFloat() == 0)
     {
        QCanvasRectangle *rect = new QCanvasRectangle(x,y,w,h,canvas());
        rect->setPen( QColor(findStyleProperty(node,"svg:stroke-color")));
        if ( findStyleProperty(node, "draw:shadow") == "visible")
        {
            QString sclr = findStyleProperty(node,"svg:shadow-color");
            QColor *shadow;
            if (sclr.length() > 0)
            {
                shadow = new QColor(sclr);
            }
            else
            {
                shadow = new QColor("#808080");  // Default Shadow Color.
            }
            QString ts = findStyleProperty(node,"draw:shadow-transparency");
            int xoff = (int)((findStyleProperty(node,"draw:shadow-offset-x").remove(findStyleProperty(node,"draw:shadow-offset-x").length()-2,2).toFloat())*CM_TO_PXL/factor);
            int yoff = (int)((findStyleProperty(node,"draw:shadow-offset-y").remove(findStyleProperty(node,"draw:shadow-offset-y").length()-2,2).toFloat())*CM_TO_PXL/factor);
            QPixmap *shadowpxmp = new QPixmap(rect->size());
            QPainter *shadowpainter = new QPainter(shadowpxmp);
            shadowpainter->setBrush(*shadow);
            shadowpainter->setPen(Qt::NoPen);
            shadowpainter->drawRect(0,0,shadowpxmp->width(),shadowpxmp->height());
            QImage i = shadowpxmp->convertToImage();
            showImage(x+xoff,y+yoff,w,h,&i,&ts);
        }
        if ( findStyleProperty(node, "draw:fill") == "solid" )
        {
            int tpcy;
            QColor *cf = new QColor(findStyleProperty(node,"draw:fill-color"));
            QString ts = findStyleProperty(node,"draw:transparency");
            if ( !ts.contains("%") )
            {
                ts = findStyleProperty(node, "draw:transparency-name");
                if (ts.length() != 0)
                {
                    QPixmap *pmp = new QPixmap(rect->size());
                    QPainter *p = new QPainter(pmp);
                    p->setBrush(*cf);
                    p->setPen(Qt::NoPen);
                    rect->setPen( QColor(findStyleProperty(node,"svg:stroke-color")));
                    p->drawRect(0,0,pmp->width(),pmp->height());
                    QImage i = pmp->convertToImage();
                    showImage(x+1,y+1,w-2,h-2,&i,&ts); 
                }  
                else
                {
                    // No transparency effect.
                    rect->setBrush(*cf);
                }
            }
            else
            {
                QPixmap *pmp = new QPixmap(rect->size());
                QPainter *p = new QPainter(pmp);
                p->setBrush(*cf);
                p->setPen(Qt::NoPen);
                rect->setPen( QColor(findStyleProperty(node,"svg:stroke-color")));
                p->drawRect(0,0,pmp->width(),pmp->height());
                QImage i = pmp->convertToImage();
                showImage(x+1,y+1,w-2,h-2,&i,&ts);
            }
        }
        else
        {
               QList<QPixmap> pml;
               QCanvasPixmap *pm;
               if ( findStyleProperty(node,"draw:fill") == "gradient" )
               {
                  QString gradname = findStyleProperty(node,"draw:fill-gradient-name");
                  pXmlNode *gradientnode = style->findChildWithNameAndValue("draw:gradient","draw:name",gradname);
                  QImage *i = makeGradientImage(gradientnode,64, new QSize(w,h));
                  int tpcy;
                  QColor *c = new QColor(findStyleProperty(node,"draw:fill-color"));
                  QString ts = findStyleProperty(node,"draw:transparency");
                  if ( !ts.contains("%") )
                  {
                     ts = findStyleProperty(node, "draw:transparency-name");
                     if  (ts.length() != 0 )
                     {
                         showImage(x+1,y+1,w-2,h-2,i,&ts);
                     }
                     else
                     {
                         showImage(x+1,y+1,w-2,h-2,i);
                     }
                  }
                  else
                  {
                       showImage(x+1,y+1,w-2,h-2,i,&ts);
                  }
               }
               else if ( findStyleProperty(node,"draw:fill") == "hatch" )
               {
                  QImage *i;
                  QString ts;
                  QString hatchname = findStyleProperty(node, "draw:fill-hatch-name");
                  pXmlNode *hatchnode = style->findChildWithNameAndValue("draw:hatch","draw:name",hatchname);
                  QColor *c = new QColor(findStyleProperty(node,"draw:fill-color"));
                  if (findStyleProperty(node,"draw:fill-hatch-solid") == "true")
                  {
                      ts = findStyleProperty(node,"draw:transparency");
                      i = makeHatchImage(hatchnode,c,0xFF,new QSize(w,h));
                      if ( !ts.contains("%") )
                      {
                         ts = findStyleProperty(node, "draw:transparency-name");
                         if  (ts.length() != 0 )
                         {
                             showImage(x+1,y+1,w-2,h-2,i,&ts);
                         }
                         else
                         {
                             showImage(x+1,y+1,w-2,h-2,i);
                         }
                      }
                      else
                      {
                           showImage(x+1,y+1,w-2,h-2,i,&ts);
                      }
                  }
                  else
                  {
                      i  = makeHatchImage(hatchnode,c,0x00,new QSize(w,h));
                      showImage(x+1,y+1,w-2,h-2,i);
                  }
               }
               else if ( findStyleProperty(node,"draw:fill") == "bitmap" )
               {
                  QString bmpname = findStyleProperty(node, "draw:fill-image-name");
                  pXmlNode *temp = style->findChildWithNameAndValue("draw:fill-image","draw:name", bmpname);
                  QString fname = temp->getA().value("xlink:href");
                  pImage *data = images.first();
                  while (((QString)"#" + data->filename) != fname)
                  {
                       data = images.next();
                       if (data == NULL)
                       {
                            continue;
                       }
                  }
                  QImage *i = data->image;
                  if (!i)
                  {
                       i = new QImage(Resource::loadImage(pUtils::getImageName(&fname)));
                  }
                  showImage(x+1,y+1,w-2,h-2,i);
               }               
        }
        rect->setZ(layer++);
        rect->show();
     }
     else
     {
        int rad = (int)(findStyleProperty(node,"draw:corner-radius").toFloat()*CM_TO_PXL/factor);
        CanvasRoundedRectangle *roundedrect;
        roundedrect = new CanvasRoundedRectangle(x,y,w,h,rad,canvas()); 
        QCanvasLine *outline;
        QPointArray linepoints = roundedrect->outlinePoints().copy();
        if ( findStyleProperty(node,"draw:fill") == "solid" )
        {
             roundedrect->setBrush( QColor( findStyleProperty(node,"draw:fill-color")));
        }
        else
        {
               // TODO
               if (replaceitems)
               {
                  roundedrect->setBrush( QColor("#DCDCDC"));
               }
        }
        roundedrect->setZ(layer++);
        roundedrect->show();

        /* QCanvasPolygons have no pen for an outline so we need to draw it ourselves */

        for (uint i = 0; i < linepoints.size(); i++ )
        {
          outline = new QCanvasLine(canvas());
          if (i < linepoints.size()-1)
          {
               outline->setPoints(linepoints.point(i).x(),linepoints.point(i).y(),linepoints.point(i+1).x(),linepoints.point(i+1).y());
          }
          else
          {
                outline->setPoints(linepoints.point(i).x(),linepoints.point(i).y(),linepoints.point(0).x(),linepoints.point(0).y());
          }
          outline->setPen(QColor(findStyleProperty(node,"svg:stroke-color")));
          outline->setZ(layer);
          outline->show();
        }
        layer++;
     }
    ((RenderTxt*)this)->render(node);
}
