/***************************************************************************
                          rendertxt.cpp  -  description
                             -------------------
    begin                : Mon Sep 2 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rendertxt.h"

RenderTxt::RenderTxt(QObject *parent, QCanvas *canvas, pXmlNode *contentroot, pXmlNode *styleroot, QList<pImage> *i)
     : RenderItems(parent, canvas, contentroot, styleroot, i)
{
}

void RenderTxt::render(pXmlNode* node, bool clearlinecount )
{
     int x,y,w,h,x2,y2,roffset=0,lastsize;

     float rtoc = 0.0;
     if (rect != NULL) // Fonts are scaled in OLE rectangles.
        rtoc = ((float)canvas()->height())/rect->height();

     if( clearlinecount )
     {
          lbcount = 0;
          lastbottom = 0;
     }

     QString algn, sz, fnt;//, bld, itlc, undrln;
     QListIterator<pXmlNode> it(node->children);
     for ( ; it.current(); ++it)
     {
            pXmlNode *c = it.current();
            QCanvasText *ctxt;
            if (c->getNodeName() == "text:p")
            {
                 if ( c->getText().length() > 0)
                 {
                      ctxt = new QCanvasText( c->getText(),canvas());
                      // Get point size of font
                      sz = findStylePropertyWithParentDelimiter(c,"draw:page","fo:font-size");
                      // Get family of font
                      fnt = findStylePropertyWithParentDelimiter(c,"draw:page","fo:font-family");
                      int size;
                      if ( rect == NULL)
                          size = (int)(sz.toFloat()/(4*factor));
                      else
                          size = (int)(sz.toFloat()/(4*factor*rtoc));
                      lastsize = size;
                      QFont f = ctxt->font();
                      f.setPointSize(size);
                      f.setFamily(fnt);
                      ctxt->setFont(f);
                      currtxt.append(ctxt);
                      roffset += ctxt->boundingRect().height();
                    /*  if (c != node->children.last())
                      {
                        currtxt.append(new QCanvasText(canvas()));
                        lastsize = -size;
                        lbcount++;
                        continue;
                      } */  
                 }
                 else
                 {
                      roffset += lastsize;
                      render(c, false);
                 }
            }
            else if (c->getNodeName() == "text:unordered-list")
            {
                 if ( c->getA().value("text:style-name").length() > 0)
                 {
                      indentnode = content->findChildWithNameAndValue("text:list-style","style:name",c->getA().value("text:style-name"));
                 }
                 render(c,false);
                 textlvl--;
                 continue;
            }
            else if (c->getNodeName() == "text:ordered-list")
            {
                 if ( c->getA().value("text:style-name").length() >0)
                 {
                      indentnode = content->findChildWithNameAndValue("text:list-style","style:name",c->getA().value("text:style-name"));
                 }
                 render(c, false);
                 textlvl--;
                 continue;
            }
            else if (c->getNodeName() == "text:list-item")
            {
                 if (!listinc)
                 {
                      currtxt.append(new QCanvasText(canvas()));
                      lbcount++;
                      listinc = true;
                 }
                 textlvl++;
                 render(c, false);
                 continue;
            }
            else if (c->getNodeName() == "text:span" )
            {
                 if ( c->getText().length() > 0)
                 {
                      ctxt = new QCanvasText( c->getText(),canvas());
                      // Get font alignment
                      algn = findStylePropertyWithParentDelimiter(c,"draw:page","fo:text-align");
                      // Get point size of font
                      sz = findStylePropertyWithParentDelimiter(c,"draw:page","fo:font-size");
                      // Get family of font
                      fnt = findStylePropertyWithParentDelimiter(c,"draw:page","fo:font-family");
                      int size;
                      if ( rect == NULL)
                          size = (int)(sz.toFloat()/(4*factor));
                      else
                          size = (int)(sz.toFloat()/(4*factor*rtoc)); 
                      lastsize = size;
                      QFont f = ctxt->font();
                      f.setPointSize(size);
                      f.setFamily(fnt);
                      ctxt->setFont(f);
                      currtxt.append(ctxt);
                      roffset += ctxt->boundingRect().height();
                      continue;
                 }
                 else if ( c->children.first()->getNodeName() == "text:line-break")
                 {
                      roffset += lastsize;
                      currtxt.append(new QCanvasText(canvas()));
                      lbcount++;
                      continue;
                 }
                 else
                 {
                      roffset += lastsize;
                      render(c, false);
                      continue;
                 }
            }
            else
            {
                 continue;
            }

            // ok now we have the QList of all the QCanvasText objects for a line, with empty text objects for new lines
            // so we can render them all at once

            if ( rect == NULL )
            {
                // rec == NULL so its a whole page being rendered.

                x = (int)((findStylePropertyWithParentDelimiter(node,"draw:page","svg:x").toFloat())*CM_TO_PXL/factor);
                y = (int)((findStylePropertyWithParentDelimiter(node,"draw:page","svg:y").toFloat())*CM_TO_PXL/factor);
                w = (int)((findStylePropertyWithParentDelimiter(node,"draw:page","svg:width").toFloat())*CM_TO_PXL/factor);
                h = (int)((findStylePropertyWithParentDelimiter(node,"draw:page","svg:height").toFloat())*CM_TO_PXL/factor);
            }
            else
            {
                // rect != NULL so we are rendering an OLE embedded section

                x = rect->left();
                y = rect->top() + roffset;
                w = rect->width();
                h = rect->height() - roffset;

                // paint it off the canvas if it is outside the bounding rect.
                // This is easier than not painting it. Maybe at some point I'll
                // set a flag or something. TODO.

                if (h < 0) y = 4096;
            }
            // Get font alignment
            algn = findStylePropertyWithParentDelimiter(node,"draw:page","fo:text-align");
            // Get vertical alignment
            QString valgn = findStylePropertyWithParentDelimiter(node,"draw:page","draw:textarea-vertical-align");
            // Get margins
            int tmargin = (int)(findStylePropertyWithParentDelimiter(node,"draw:page","fo:margin-top").toFloat()*(CM_TO_PXL/factor));
            int bmargin = (int)(findStylePropertyWithParentDelimiter(node,"draw:page","fo:margin-bottom").toFloat()*(CM_TO_PXL/factor));
            int lmargin = (int)(findStylePropertyWithParentDelimiter(node,"draw:page","fo:margin-left").toFloat()*(CM_TO_PXL/factor));
            int rmargin = (int)(findStylePropertyWithParentDelimiter(node,"draw:page","fo:margin-right").toFloat()*(CM_TO_PXL/factor));
            int listindent = 0;
            roffset += bmargin + tmargin;
            QString lnp = "";
            QCanvasEllipse *bullet = NULL;
            QCanvasText *textbullet = NULL;
            QCanvasLine *linebullet = NULL;
            QCanvasSprite *imagebullet = NULL;
            if (listinc)
            {
                 QString q = "";
                 q.setNum(textlvl);
                 pXmlNode *tnode = indentnode;
                 indentnode = indentnode->findChildWithNameAndValue("text:list-level-style-bullet","text:level", q);
                 if ( indentnode == NULL )
                 {
                      indentnode = tnode->findChildWithNameAndValue("text:list-level-style-number","text:level", q);
                      if ( indentnode )
                      {

                           lnp = findStyleProperty(indentnode,"style:num-prefix");
                           QString nf = findStyleProperty(indentnode,"style:num-format");
                           if ( nf == "a")
                           {
                                lnp += (char)(lbcount+'a'-1);
                           }
                           else if (nf == "A")
                           {
                                lnp += (char)(lbcount+'A'-1);
                           }
                           else if ( nf == "1")
                           {
                                QString w = "";
                                w.setNum(lbcount);
                                lnp += w;
                           }
                           else if ( nf == "i")
                           {
                                lnp += pUtils::roman(lbcount,false);
                           }
                           else if ( nf == "I")
                           {
                                lnp += pUtils::roman(lbcount,true);
                           }
                           lnp += findStyleProperty(indentnode,"style:num-suffix");
                           listindent = (int)(indentnode->children.first()->getA().value("text:space-before").toFloat()*(CM_TO_PXL/factor));
                           QFont tf;
                           float fosize = indentnode->children.first()->getA().value("fo:font-size").toFloat()/100;
                           tf.setPointSize((int)(fosize*sz.toFloat()/(3.5*factor)));
                           tf.setFamily(fnt);
                           textbullet = new QCanvasText(lnp,tf,canvas());
                      }
                      else
                      {
                           indentnode = tnode->findChildWithNameAndValue("text:list-level-style-image","text:level", q);
                           if (indentnode)
                           {
                                QString href = findStyleProperty(indentnode,"xlink:href");
                                pImage *data = images.first();
                                while (((QString)"#" + data->filename) != href)
                                {
                                     data = images.next();
                                     if (data == NULL)
                                     {
                                          continue;
                                     }
                                }
                                QImage *i = data->image;
                                if (!i)
                                {
                                     i = new QImage(Resource::loadImage(pUtils::getImageName(&href)));
                                     i->setAlphaBuffer(TRUE);
                                     QList<QPixmap> pml;
                                     int w = (int)(indentnode->children.first()->getA().value("fo:width").toFloat()*(CM_TO_PXL/factor));
                                     int h = (int)(indentnode->children.first()->getA().value("fo:height").toFloat()*(CM_TO_PXL/factor));
                                     pml.append(new QCanvasPixmap(i->smoothScale(w+4,h+4)));
                                     QList<QPoint> ptl;
                                     ptl.append(new QPoint(0,0));
                                     QCanvasPixmapArray *pma = new QCanvasPixmapArray(pml, ptl);
                                     imagebullet = new QCanvasSprite(pma,canvas()); 
                                }
                                else
                                {
                                     i->setAlphaBuffer(TRUE);
                                     QList<QPixmap> pml;
                                     int w = (int)(indentnode->children.first()->getA().value("fo:width").toFloat()*(CM_TO_PXL/factor));
                                     int h = (int)(indentnode->children.first()->getA().value("fo:height").toFloat()*(CM_TO_PXL/factor));
                                     pml.append(new QCanvasPixmap(i->smoothScale(w+4,h+4)));
                                     QList<QPoint> ptl;
                                     ptl.append(new QPoint(0,0));
                                     QCanvasPixmapArray *pma = new QCanvasPixmapArray(pml, ptl);
                                     imagebullet = new QCanvasSprite(pma,canvas());
                                }
                           }
                      }
                 }
                 else
                 {
                      listindent = (int)(indentnode->children.first()->getA().value("text:space-before").toFloat()*(CM_TO_PXL/factor));
                      float fosize = indentnode->children.first()->getA().value("fo:font-size").toFloat()/100;
                      fosize = fosize*sz.toFloat()/(3.5*factor);
                      if ((textlvl > 4) || (textlvl%2 == 1))
                      {
                           bullet = new QCanvasEllipse((int)(fosize),(int)(fosize),canvas());
                           bullet->setBrush( QColor("black"));
                      }
                      else
                      {
                           linebullet = new QCanvasLine(canvas());
                           linebullet->setPoints(0,0,(int)(fosize),0);
                      }
                 }
            }
            int linelg = 0, lineh = 0;
            int currx = 0, curry = 0;
            QListIterator<QCanvasText> newit(currtxt);
            for ( ; newit.current(); ++newit)
            {
                   // empty lines have text() = "<text>" ... I dont know why.
                   
                   if ( newit.current()->text() !=  "<text>" )
                   {
                        currline.append(newit.current());
                   }
                   else
                   {
                        //print out currline
                        QListIterator<QCanvasText> nit(currline);
                        for ( ; nit.current(); ++nit)
                        {
                             linelg += nit.current()->boundingRect().width();
                             if ( nit.current()->boundingRect().height() > lineh )
                                  lineh = nit.current()->boundingRect().height();
                        }
                        if (( algn == "center" ) || ( node->getNodeName() == "draw:rect") ||  // apparently non text boxes dont care
                           ( node->getNodeName() == "draw:ellipse") ||
                           ( node->getNodeName() == "draw:circle" ) ||
                           ( node->getNodeName() == "draw:image"))
                        {
                             x2 = (w - lmargin - rmargin - linelg)/2;
                        }
                        else if ( algn == "end" )
                        {
                             x2 = (w - rmargin - linelg);
                        }
                        else
                        {
                             x2 = lmargin;
                        }
                        if (valgn == "middle")
                             y2 = (h - (tmargin + bmargin + lineh)*(lbcount+1))/2;
                        else
                             y2 = (bmargin+lineh)*(lbcount-1);
                        nit.toFirst();
                        for ( ; nit.current(); ++nit)
                        {
                            QCanvasText *t = nit.current();
                            t->move(x+x2+currx,y+y2+curry);
                            t->setZ(layer++);
                            t->show();
                            if (bullet)
                            {
                                 bullet->move(t->x()-lmargin,t->y()+lineh/2);
                                 bullet->setZ(layer++);
                                 bullet->show();
                            }
                            else if (linebullet)
                            {
                                 linebullet->move(t->x()-lmargin,t->y()+lineh/2);
                                 linebullet->setZ(layer++);
                                 linebullet->show();
                            }
                            else if (textbullet)
                            {
                                 textbullet->move(t->x()-lmargin,t->y());
                                 textbullet->setZ(layer++);
                                 textbullet->show();
                            }
                            else if (imagebullet)
                            {
                                 imagebullet->move(t->x()-lmargin,t->y()+(lineh-imagebullet->boundingRect().height())/2);
                                 imagebullet->setZ(layer++);
                                 imagebullet->show();
                            }
                            currx += t->boundingRect().width();
                        }
                        curry += lineh;
                        currline.clear();
                        linelg = 0;
                        currx = 0;
                   }
            }
            //print out currline and clear buffer
            QListIterator<QCanvasText> nit(currline);
            for ( ; nit.current(); ++nit)
            {
                 linelg += nit.current()->boundingRect().width();
                 if ( nit.current()->boundingRect().height() > lineh )
                      lineh = nit.current()->boundingRect().height();
            }
            if (( algn == "center" ) || ( node->getNodeName() == "draw:rect") ||  // apparently non text boxes dont care
               ( node->getNodeName() == "draw:ellipse") ||
               ( node->getNodeName() == "draw:circle" ) ||
               ( node->getNodeName() == "draw:image"))
            {
                 x2 = (w - lmargin - rmargin - linelg)/2;
            }
            else if ( algn == "right" )
            {
                 x2 = (w - rmargin - linelg);
            }
            else
            {
                 x2 = lmargin;
            }
            if (valgn == "middle")
                 y2 = (h - (tmargin + bmargin+ lineh)*(lbcount+1))/2;
            else
                 y2 = (lineh+bmargin)*(lbcount-1);
            nit.toFirst();
            for ( ; nit.current(); ++nit)
            {
                QCanvasText *t = nit.current();
                t->move(x+x2+currx+listindent,y+y2+curry);
                t->setZ(layer++);
                t->show();
                if (bullet)
                {
                     bullet->move(t->x()-lmargin,t->y()+lineh/2);
                     bullet->setZ(layer++);
                     bullet->show();
                }
                else if (linebullet)
                {
                     linebullet->move(t->x()-lmargin,t->y()+lineh/2);
                     linebullet->setZ(layer++);
                     linebullet->show();
                }
                else if (textbullet)
                {
                     textbullet->move(t->x()-lmargin,t->y());
                     textbullet->setZ(layer++);
                     textbullet->show();
                }
                else if (imagebullet)
                {
                     imagebullet->move(t->x()-lmargin,t->y()+(lineh-imagebullet->boundingRect().height())/2);
                     imagebullet->setZ(layer++);
                     imagebullet->show();
                }
                currx += t->boundingRect().width();
            }
            currtxt.clear();
            currline.clear();
            lastbottom = bmargin;
            listinc = false;
            currx = 0;
     }
}